# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=useless-super-delegation

import datetime
from typing import Any, Mapping, Optional, TYPE_CHECKING, Union, overload

from azure.core.exceptions import ODataV4Format

from .._utils.model_base import Model as _Model, rest_field
from .._utils.utils import FileType

if TYPE_CHECKING:
    from .. import models as _models


class ActiveLearningFeedback(_Model):
    """Feedback for Active Learning.

    :ivar records: A list of Feedback Records for Active Learning.
    :vartype records: list[~azure.ai.language.questionanswering.authoring.models.FeedbackRecord]
    """

    records: Optional[list["_models.FeedbackRecord"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """A list of Feedback Records for Active Learning."""

    @overload
    def __init__(
        self,
        *,
        records: Optional[list["_models.FeedbackRecord"]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class Assets(_Model):
    """All assets for this project.

    :ivar synonyms: Collection of synonyms.
    :vartype synonyms: list[~azure.ai.language.questionanswering.authoring.models.WordAlterations]
    :ivar qnas: List of QnA records to import.
    :vartype qnas: list[~azure.ai.language.questionanswering.authoring.models.ImportQnaRecord]
    """

    synonyms: Optional[list["_models.WordAlterations"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Collection of synonyms."""
    qnas: Optional[list["_models.ImportQnaRecord"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """List of QnA records to import."""

    @overload
    def __init__(
        self,
        *,
        synonyms: Optional[list["_models.WordAlterations"]] = None,
        qnas: Optional[list["_models.ImportQnaRecord"]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class Error(_Model):
    """The error object.

    :ivar code: One of a server-defined set of error codes. Required. Known values are:
     "InvalidRequest", "InvalidArgument", "Unauthorized", "Forbidden", "NotFound",
     "ProjectNotFound", "OperationNotFound", "AzureCognitiveSearchNotFound",
     "AzureCognitiveSearchIndexNotFound", "TooManyRequests", "AzureCognitiveSearchThrottling",
     "AzureCognitiveSearchIndexLimitReached", "InternalServerError", "ServiceUnavailable",
     "Timeout", "QuotaExceeded", "Conflict", and "Warning".
    :vartype code: str or ~azure.ai.language.questionanswering.authoring.models.ErrorCode
    :ivar message: A human-readable representation of the error. Required.
    :vartype message: str
    :ivar target: The target of the error.
    :vartype target: str
    :ivar details: An array of details about specific errors that led to this reported error.
    :vartype details: list[~azure.ai.language.questionanswering.authoring.models.Error]
    :ivar innererror: An object containing more specific information than the current object about
     the error.
    :vartype innererror: ~azure.ai.language.questionanswering.authoring.models.InnerErrorModel
    """

    code: Union[str, "_models.ErrorCode"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """One of a server-defined set of error codes. Required. Known values are: \"InvalidRequest\",
     \"InvalidArgument\", \"Unauthorized\", \"Forbidden\", \"NotFound\", \"ProjectNotFound\",
     \"OperationNotFound\", \"AzureCognitiveSearchNotFound\", \"AzureCognitiveSearchIndexNotFound\",
     \"TooManyRequests\", \"AzureCognitiveSearchThrottling\",
     \"AzureCognitiveSearchIndexLimitReached\", \"InternalServerError\", \"ServiceUnavailable\",
     \"Timeout\", \"QuotaExceeded\", \"Conflict\", and \"Warning\"."""
    message: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """A human-readable representation of the error. Required."""
    target: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The target of the error."""
    details: Optional[list["_models.Error"]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """An array of details about specific errors that led to this reported error."""
    innererror: Optional["_models.InnerErrorModel"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """An object containing more specific information than the current object about
     the error."""

    @overload
    def __init__(
        self,
        *,
        code: Union[str, "_models.ErrorCode"],
        message: str,
        target: Optional[str] = None,
        details: Optional[list["_models.Error"]] = None,
        innererror: Optional["_models.InnerErrorModel"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class ErrorResponse(_Model):
    """Error response.

    :ivar error: The error object. Required.
    :vartype error: ~azure.ai.language.questionanswering.authoring.models.Error
    """

    error: "_models.Error" = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The error object. Required."""

    @overload
    def __init__(
        self,
        *,
        error: "_models.Error",
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class FeedbackRecord(_Model):
    """Feedback Record for Active Learning.

    :ivar user_id: Unique identifier of the user.
    :vartype user_id: str
    :ivar user_question: User suggested question for the QnA.
    :vartype user_question: str
    :ivar qna_id: Unique ID of the QnA.
    :vartype qna_id: int
    """

    user_id: Optional[str] = rest_field(name="userId", visibility=["read", "create", "update", "delete", "query"])
    """Unique identifier of the user."""
    user_question: Optional[str] = rest_field(
        name="userQuestion", visibility=["read", "create", "update", "delete", "query"]
    )
    """User suggested question for the QnA."""
    qna_id: Optional[int] = rest_field(name="qnaId", visibility=["read", "create", "update", "delete", "query"])
    """Unique ID of the QnA."""

    @overload
    def __init__(
        self,
        *,
        user_id: Optional[str] = None,
        user_question: Optional[str] = None,
        qna_id: Optional[int] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class ImportFiles(_Model):
    """Collection of files containing project assets that need to be imported.

    :ivar files: Collection of files where the fileName is required. Required.
    :vartype files: list[~azure.ai.language.questionanswering.authoring._utils.utils.FileType]
    """

    files: list[FileType] = rest_field(
        visibility=["read", "create", "update", "delete", "query"], is_multipart_file_input=True
    )
    """Collection of files where the fileName is required. Required."""

    @overload
    def __init__(
        self,
        *,
        files: list[FileType],
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class ImportJobOptions(_Model):
    """Project assets that need to be imported.

    :ivar metadata: Parameters needed to create the project.
    :vartype metadata:
     ~azure.ai.language.questionanswering.authoring.models.QuestionAnsweringProject
    :ivar assets: All assets for this project.
    :vartype assets: ~azure.ai.language.questionanswering.authoring.models.Assets
    :ivar file_uri: Import data File URI.
    :vartype file_uri: str
    """

    metadata: Optional["_models.QuestionAnsweringProject"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Parameters needed to create the project."""
    assets: Optional["_models.Assets"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """All assets for this project."""
    file_uri: Optional[str] = rest_field(name="fileUri", visibility=["read", "create", "update", "delete", "query"])
    """Import data File URI."""

    @overload
    def __init__(
        self,
        *,
        metadata: Optional["_models.QuestionAnsweringProject"] = None,
        assets: Optional["_models.Assets"] = None,
        file_uri: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class ImportQnaRecord(_Model):
    """Import QnA Record.

    :ivar id: Unique ID for the QnA. Required.
    :vartype id: int
    :ivar answer: Answer text.
    :vartype answer: str
    :ivar source: Source from which QnA was indexed e.g.
     `https://docs.microsoft.com/en-us/azure/cognitive-services/QnAMaker/FAQs
     <https://docs.microsoft.com/en-us/azure/cognitive-services/QnAMaker/FAQs>`_ .
    :vartype source: str
    :ivar questions: List of questions associated with the answer.
    :vartype questions: list[str]
    :ivar metadata: Metadata associated with the answer, useful to categorize or filter question
     answers.
    :vartype metadata: dict[str, str]
    :ivar dialog: Context of a QnA.
    :vartype dialog: ~azure.ai.language.questionanswering.authoring.models.QnaDialog
    :ivar active_learning_suggestions: List of Active Learning suggestions for the QnA.
    :vartype active_learning_suggestions:
     list[~azure.ai.language.questionanswering.authoring.models.SuggestedQuestionsCluster]
    :ivar last_updated_date_time: Date-time when the QnA was last updated.
    :vartype last_updated_date_time: ~datetime.datetime
    :ivar source_display_name: Friendly name of the Source.
    :vartype source_display_name: str
    """

    id: int = rest_field(visibility=["read", "create"])
    """Unique ID for the QnA. Required."""
    answer: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Answer text."""
    source: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Source from which QnA was indexed e.g.
     `https://docs.microsoft.com/en-us/azure/cognitive-services/QnAMaker/FAQs
     <https://docs.microsoft.com/en-us/azure/cognitive-services/QnAMaker/FAQs>`_ ."""
    questions: Optional[list[str]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """List of questions associated with the answer."""
    metadata: Optional[dict[str, str]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Metadata associated with the answer, useful to categorize or filter question
     answers."""
    dialog: Optional["_models.QnaDialog"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Context of a QnA."""
    active_learning_suggestions: Optional[list["_models.SuggestedQuestionsCluster"]] = rest_field(
        name="activeLearningSuggestions", visibility=["read", "create", "update", "delete", "query"]
    )
    """List of Active Learning suggestions for the QnA."""
    last_updated_date_time: Optional[datetime.datetime] = rest_field(
        name="lastUpdatedDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """Date-time when the QnA was last updated."""
    source_display_name: Optional[str] = rest_field(
        name="sourceDisplayName", visibility=["read", "create", "update", "delete", "query"]
    )
    """Friendly name of the Source."""

    @overload
    def __init__(
        self,
        *,
        id: int,  # pylint: disable=redefined-builtin
        answer: Optional[str] = None,
        source: Optional[str] = None,
        questions: Optional[list[str]] = None,
        metadata: Optional[dict[str, str]] = None,
        dialog: Optional["_models.QnaDialog"] = None,
        active_learning_suggestions: Optional[list["_models.SuggestedQuestionsCluster"]] = None,
        last_updated_date_time: Optional[datetime.datetime] = None,
        source_display_name: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class InnerErrorModel(_Model):
    """An object containing more specific information about the error. As per
    Microsoft One API guidelines -
    `https://github.com/Microsoft/api-guidelines/blob/vNext/Guidelines.md#7102-error-condition-responses
    <https://github.com/Microsoft/api-guidelines/blob/vNext/Guidelines.md#7102-error-condition-responses>`_.

    :ivar code: One of a server-defined set of error codes. Required. Known values are:
     "InvalidRequest", "InvalidParameterValue", "KnowledgeBaseNotFound",
     "AzureCognitiveSearchNotFound", "AzureCognitiveSearchThrottling", "ExtractionFailure",
     "InvalidRequestBodyFormat", "EmptyRequest", "MissingInputDocuments", "InvalidDocument",
     "ModelVersionIncorrect", "InvalidDocumentBatch", "UnsupportedLanguageCode", and
     "InvalidCountryHint".
    :vartype code: str or ~azure.ai.language.questionanswering.authoring.models.InnerErrorCode
    :ivar message: Error message. Required.
    :vartype message: str
    :ivar details: Error details.
    :vartype details: dict[str, str]
    :ivar target: Error target.
    :vartype target: str
    :ivar innererror: An object containing more specific information than the current object about
     the error.
    :vartype innererror: ~azure.ai.language.questionanswering.authoring.models.InnerErrorModel
    """

    code: Union[str, "_models.InnerErrorCode"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """One of a server-defined set of error codes. Required. Known values are: \"InvalidRequest\",
     \"InvalidParameterValue\", \"KnowledgeBaseNotFound\", \"AzureCognitiveSearchNotFound\",
     \"AzureCognitiveSearchThrottling\", \"ExtractionFailure\", \"InvalidRequestBodyFormat\",
     \"EmptyRequest\", \"MissingInputDocuments\", \"InvalidDocument\", \"ModelVersionIncorrect\",
     \"InvalidDocumentBatch\", \"UnsupportedLanguageCode\", and \"InvalidCountryHint\"."""
    message: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Error message. Required."""
    details: Optional[dict[str, str]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Error details."""
    target: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Error target."""
    innererror: Optional["_models.InnerErrorModel"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """An object containing more specific information than the current object about
     the error."""

    @overload
    def __init__(
        self,
        *,
        code: Union[str, "_models.InnerErrorCode"],
        message: str,
        details: Optional[dict[str, str]] = None,
        target: Optional[str] = None,
        innererror: Optional["_models.InnerErrorModel"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class ProjectDeployment(_Model):
    """Project deployment details.

    :ivar deployment_name: Name of the deployment. Required.
    :vartype deployment_name: str
    :ivar last_deployed_date_time: Represents the project last deployment date-time.
    :vartype last_deployed_date_time: ~datetime.datetime
    """

    deployment_name: str = rest_field(name="deploymentName", visibility=["read"])
    """Name of the deployment. Required."""
    last_deployed_date_time: Optional[datetime.datetime] = rest_field(
        name="lastDeployedDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """Represents the project last deployment date-time."""

    @overload
    def __init__(
        self,
        *,
        last_deployed_date_time: Optional[datetime.datetime] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class ProjectSettings(_Model):
    """Configurable settings of the Project.

    :ivar default_answer: Default Answer response when no good match is found in the knowledge
     base.
    :vartype default_answer: str
    """

    default_answer: Optional[str] = rest_field(
        name="defaultAnswer", visibility=["read", "create", "update", "delete", "query"]
    )
    """Default Answer response when no good match is found in the knowledge base."""

    @overload
    def __init__(
        self,
        *,
        default_answer: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class QnaDialog(_Model):
    """Dialog associated with QnA Record.

    :ivar is_context_only: To mark if a prompt is relevant only with a previous question or not. If
     true,
     do not include this QnA as answer for queries without context; otherwise,
     ignores context and includes this QnA in answers.
    :vartype is_context_only: bool
    :ivar prompts: List of prompts associated with the answer.
    :vartype prompts: list[~azure.ai.language.questionanswering.authoring.models.QnaPrompt]
    """

    is_context_only: Optional[bool] = rest_field(
        name="isContextOnly", visibility=["read", "create", "update", "delete", "query"]
    )
    """To mark if a prompt is relevant only with a previous question or not. If true,
     do not include this QnA as answer for queries without context; otherwise,
     ignores context and includes this QnA in answers."""
    prompts: Optional[list["_models.QnaPrompt"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """List of prompts associated with the answer."""

    @overload
    def __init__(
        self,
        *,
        is_context_only: Optional[bool] = None,
        prompts: Optional[list["_models.QnaPrompt"]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class QnaPrompt(_Model):
    """Prompt for an answer.

    :ivar display_order: Index of the prompt. It is used for ordering of the prompts.
    :vartype display_order: int
    :ivar qna_id: ID of the QnA corresponding to the prompt.
    :vartype qna_id: int
    :ivar qna: QnA record. Either QnAId or QnA record needs to be present in a Prompt.
    :vartype qna: ~azure.ai.language.questionanswering.authoring.models.QnaRecord
    :ivar display_text: Text displayed to represent a follow up question prompt.
    :vartype display_text: str
    """

    display_order: Optional[int] = rest_field(
        name="displayOrder", visibility=["read", "create", "update", "delete", "query"]
    )
    """Index of the prompt. It is used for ordering of the prompts."""
    qna_id: Optional[int] = rest_field(name="qnaId", visibility=["read", "create", "update", "delete", "query"])
    """ID of the QnA corresponding to the prompt."""
    qna: Optional["_models.QnaRecord"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """QnA record. Either QnAId or QnA record needs to be present in a Prompt."""
    display_text: Optional[str] = rest_field(
        name="displayText", visibility=["read", "create", "update", "delete", "query"]
    )
    """Text displayed to represent a follow up question prompt."""

    @overload
    def __init__(
        self,
        *,
        display_order: Optional[int] = None,
        qna_id: Optional[int] = None,
        qna: Optional["_models.QnaRecord"] = None,
        display_text: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class QnaRecord(_Model):
    """QnA record.

    :ivar id: Unique ID for the QnA. Required.
    :vartype id: int
    :ivar answer: Answer text.
    :vartype answer: str
    :ivar source: Source from which QnA was indexed e.g.
     `https://docs.microsoft.com/en-us/azure/cognitive-services/QnAMaker/FAQs
     <https://docs.microsoft.com/en-us/azure/cognitive-services/QnAMaker/FAQs>`_ .
    :vartype source: str
    :ivar questions: List of questions associated with the answer.
    :vartype questions: list[str]
    :ivar metadata: Metadata associated with the answer, useful to categorize or filter question
     answers.
    :vartype metadata: dict[str, str]
    :ivar dialog: Context of a QnA.
    :vartype dialog: ~azure.ai.language.questionanswering.authoring.models.QnaDialog
    :ivar active_learning_suggestions: List of Active Learning suggestions for the QnA.
    :vartype active_learning_suggestions:
     list[~azure.ai.language.questionanswering.authoring.models.SuggestedQuestionsCluster]
    """

    id: int = rest_field(visibility=["read", "create"])
    """Unique ID for the QnA. Required."""
    answer: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Answer text."""
    source: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Source from which QnA was indexed e.g.
     `https://docs.microsoft.com/en-us/azure/cognitive-services/QnAMaker/FAQs
     <https://docs.microsoft.com/en-us/azure/cognitive-services/QnAMaker/FAQs>`_ ."""
    questions: Optional[list[str]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """List of questions associated with the answer."""
    metadata: Optional[dict[str, str]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Metadata associated with the answer, useful to categorize or filter question
     answers."""
    dialog: Optional["_models.QnaDialog"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Context of a QnA."""
    active_learning_suggestions: Optional[list["_models.SuggestedQuestionsCluster"]] = rest_field(
        name="activeLearningSuggestions", visibility=["read", "create", "update", "delete", "query"]
    )
    """List of Active Learning suggestions for the QnA."""

    @overload
    def __init__(
        self,
        *,
        id: int,  # pylint: disable=redefined-builtin
        answer: Optional[str] = None,
        source: Optional[str] = None,
        questions: Optional[list[str]] = None,
        metadata: Optional[dict[str, str]] = None,
        dialog: Optional["_models.QnaDialog"] = None,
        active_learning_suggestions: Optional[list["_models.SuggestedQuestionsCluster"]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class QnaSourceRecord(_Model):
    """Custom source record with last updated date-time.

    :ivar display_name: Friendly name of the Source.
    :vartype display_name: str
    :ivar source: Unique source identifier. Name of the file if it's a 'file' source; otherwise,
     the complete URL if it's a 'url' source. Required.
    :vartype source: str
    :ivar source_uri: URI location for the file or url. Required.
    :vartype source_uri: str
    :ivar source_kind: Supported source types. Required. Known values are: "file" and "url".
    :vartype source_kind: str or ~azure.ai.language.questionanswering.authoring.models.SourceKind
    :ivar content_structure_kind: Content structure type for sources. "unstructured"
    :vartype content_structure_kind: str or
     ~azure.ai.language.questionanswering.authoring.models.SourceContentStructureKind
    :ivar last_updated_date_time: Date-time when the QnA was last updated.
    :vartype last_updated_date_time: ~datetime.datetime
    """

    display_name: Optional[str] = rest_field(
        name="displayName", visibility=["read", "create", "update", "delete", "query"]
    )
    """Friendly name of the Source."""
    source: str = rest_field(visibility=["read", "create", "update"])
    """Unique source identifier. Name of the file if it's a 'file' source; otherwise,
     the complete URL if it's a 'url' source. Required."""
    source_uri: str = rest_field(name="sourceUri", visibility=["read", "create", "update", "delete", "query"])
    """URI location for the file or url. Required."""
    source_kind: Union[str, "_models.SourceKind"] = rest_field(
        name="sourceKind", visibility=["read", "create", "update", "delete", "query"]
    )
    """Supported source types. Required. Known values are: \"file\" and \"url\"."""
    content_structure_kind: Optional[Union[str, "_models.SourceContentStructureKind"]] = rest_field(
        name="contentStructureKind", visibility=["read", "create", "update", "delete", "query"]
    )
    """Content structure type for sources. \"unstructured\""""
    last_updated_date_time: Optional[datetime.datetime] = rest_field(
        name="lastUpdatedDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """Date-time when the QnA was last updated."""

    @overload
    def __init__(
        self,
        *,
        source: str,
        source_uri: str,
        source_kind: Union[str, "_models.SourceKind"],
        display_name: Optional[str] = None,
        content_structure_kind: Optional[Union[str, "_models.SourceContentStructureKind"]] = None,
        last_updated_date_time: Optional[datetime.datetime] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class QuestionAnsweringAuthoringExportJobState(_Model):
    """Export job status, project metadata, and assets.

    :ivar created_date_time: The creation date time of the job. Required.
    :vartype created_date_time: ~datetime.datetime
    :ivar expiration_date_time: The expiration date time of the job.
    :vartype expiration_date_time: ~datetime.datetime
    :ivar job_id: The job ID. Required.
    :vartype job_id: str
    :ivar last_updated_date_time: The last date time the job was updated. Required.
    :vartype last_updated_date_time: ~datetime.datetime
    :ivar status: Job Status. Required. Known values are: "notStarted", "running", "succeeded",
     "failed", "cancelled", "cancelling", and "partiallyCompleted".
    :vartype status: str or ~azure.ai.language.questionanswering.authoring.models.JobStatus
    :ivar errors: The errors encountered while executing the job.
    :vartype errors: list[~azure.core.ODataV4Format]
    :ivar result_url: URL to download the result of the Export Job. Required.
    :vartype result_url: str
    """

    created_date_time: datetime.datetime = rest_field(
        name="createdDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The creation date time of the job. Required."""
    expiration_date_time: Optional[datetime.datetime] = rest_field(
        name="expirationDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The expiration date time of the job."""
    job_id: str = rest_field(name="jobId", visibility=["read"])
    """The job ID. Required."""
    last_updated_date_time: datetime.datetime = rest_field(
        name="lastUpdatedDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The last date time the job was updated. Required."""
    status: Union[str, "_models.JobStatus"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Job Status. Required. Known values are: \"notStarted\", \"running\", \"succeeded\", \"failed\",
     \"cancelled\", \"cancelling\", and \"partiallyCompleted\"."""
    errors: Optional[list[ODataV4Format]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The errors encountered while executing the job."""
    result_url: str = rest_field(name="resultUrl", visibility=["read", "create", "update", "delete", "query"])
    """URL to download the result of the Export Job. Required."""

    @overload
    def __init__(
        self,
        *,
        created_date_time: datetime.datetime,
        last_updated_date_time: datetime.datetime,
        status: Union[str, "_models.JobStatus"],
        result_url: str,
        expiration_date_time: Optional[datetime.datetime] = None,
        errors: Optional[list[ODataV4Format]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class QuestionAnsweringAuthoringImportJobState(_Model):
    """Import job status, project metadata, and assets.

    :ivar created_date_time: The creation date time of the job. Required.
    :vartype created_date_time: ~datetime.datetime
    :ivar expiration_date_time: The expiration date time of the job.
    :vartype expiration_date_time: ~datetime.datetime
    :ivar job_id: The job ID. Required.
    :vartype job_id: str
    :ivar last_updated_date_time: The last date time the job was updated. Required.
    :vartype last_updated_date_time: ~datetime.datetime
    :ivar status: Job Status. Required. Known values are: "notStarted", "running", "succeeded",
     "failed", "cancelled", "cancelling", and "partiallyCompleted".
    :vartype status: str or ~azure.ai.language.questionanswering.authoring.models.JobStatus
    :ivar errors: The errors encountered while executing the job.
    :vartype errors: list[~azure.core.ODataV4Format]
    """

    created_date_time: datetime.datetime = rest_field(
        name="createdDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The creation date time of the job. Required."""
    expiration_date_time: Optional[datetime.datetime] = rest_field(
        name="expirationDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The expiration date time of the job."""
    job_id: str = rest_field(name="jobId", visibility=["read"])
    """The job ID. Required."""
    last_updated_date_time: datetime.datetime = rest_field(
        name="lastUpdatedDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The last date time the job was updated. Required."""
    status: Union[str, "_models.JobStatus"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Job Status. Required. Known values are: \"notStarted\", \"running\", \"succeeded\", \"failed\",
     \"cancelled\", \"cancelling\", and \"partiallyCompleted\"."""
    errors: Optional[list[ODataV4Format]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The errors encountered while executing the job."""

    @overload
    def __init__(
        self,
        *,
        created_date_time: datetime.datetime,
        last_updated_date_time: datetime.datetime,
        status: Union[str, "_models.JobStatus"],
        expiration_date_time: Optional[datetime.datetime] = None,
        errors: Optional[list[ODataV4Format]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class QuestionAnsweringAuthoringProjectDeletionJobState(_Model):  # pylint: disable=name-too-long
    """Represents the state of a project deletion job.

    :ivar created_date_time: The creation date time of the job. Required.
    :vartype created_date_time: ~datetime.datetime
    :ivar expiration_date_time: The expiration date time of the job.
    :vartype expiration_date_time: ~datetime.datetime
    :ivar job_id: The job ID. Required.
    :vartype job_id: str
    :ivar last_updated_date_time: The last date time the job was updated. Required.
    :vartype last_updated_date_time: ~datetime.datetime
    :ivar status: Job Status. Required. Known values are: "notStarted", "running", "succeeded",
     "failed", "cancelled", "cancelling", and "partiallyCompleted".
    :vartype status: str or ~azure.ai.language.questionanswering.authoring.models.JobStatus
    :ivar errors: The errors encountered while executing the job.
    :vartype errors: list[~azure.core.ODataV4Format]
    """

    created_date_time: datetime.datetime = rest_field(
        name="createdDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The creation date time of the job. Required."""
    expiration_date_time: Optional[datetime.datetime] = rest_field(
        name="expirationDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The expiration date time of the job."""
    job_id: str = rest_field(name="jobId", visibility=["read"])
    """The job ID. Required."""
    last_updated_date_time: datetime.datetime = rest_field(
        name="lastUpdatedDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The last date time the job was updated. Required."""
    status: Union[str, "_models.JobStatus"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Job Status. Required. Known values are: \"notStarted\", \"running\", \"succeeded\", \"failed\",
     \"cancelled\", \"cancelling\", and \"partiallyCompleted\"."""
    errors: Optional[list[ODataV4Format]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The errors encountered while executing the job."""

    @overload
    def __init__(
        self,
        *,
        created_date_time: datetime.datetime,
        last_updated_date_time: datetime.datetime,
        status: Union[str, "_models.JobStatus"],
        expiration_date_time: Optional[datetime.datetime] = None,
        errors: Optional[list[ODataV4Format]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class QuestionAnsweringAuthoringProjectDeploymentJobState(_Model):  # pylint: disable=name-too-long
    """Job state represents the job metadata and any errors.

    :ivar created_date_time: The creation date time of the job. Required.
    :vartype created_date_time: ~datetime.datetime
    :ivar expiration_date_time: The expiration date time of the job.
    :vartype expiration_date_time: ~datetime.datetime
    :ivar job_id: The job ID. Required.
    :vartype job_id: str
    :ivar last_updated_date_time: The last date time the job was updated. Required.
    :vartype last_updated_date_time: ~datetime.datetime
    :ivar status: Job Status. Required. Known values are: "notStarted", "running", "succeeded",
     "failed", "cancelled", "cancelling", and "partiallyCompleted".
    :vartype status: str or ~azure.ai.language.questionanswering.authoring.models.JobStatus
    :ivar errors: The errors encountered while executing the job.
    :vartype errors: list[~azure.core.ODataV4Format]
    """

    created_date_time: datetime.datetime = rest_field(
        name="createdDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The creation date time of the job. Required."""
    expiration_date_time: Optional[datetime.datetime] = rest_field(
        name="expirationDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The expiration date time of the job."""
    job_id: str = rest_field(name="jobId", visibility=["read"])
    """The job ID. Required."""
    last_updated_date_time: datetime.datetime = rest_field(
        name="lastUpdatedDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The last date time the job was updated. Required."""
    status: Union[str, "_models.JobStatus"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Job Status. Required. Known values are: \"notStarted\", \"running\", \"succeeded\", \"failed\",
     \"cancelled\", \"cancelling\", and \"partiallyCompleted\"."""
    errors: Optional[list[ODataV4Format]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The errors encountered while executing the job."""

    @overload
    def __init__(
        self,
        *,
        created_date_time: datetime.datetime,
        last_updated_date_time: datetime.datetime,
        status: Union[str, "_models.JobStatus"],
        expiration_date_time: Optional[datetime.datetime] = None,
        errors: Optional[list[ODataV4Format]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class QuestionAnsweringAuthoringUpdateQnasJobState(_Model):  # pylint: disable=name-too-long
    """Job state represents the job metadata and any errors.

    :ivar created_date_time: The creation date time of the job. Required.
    :vartype created_date_time: ~datetime.datetime
    :ivar expiration_date_time: The expiration date time of the job.
    :vartype expiration_date_time: ~datetime.datetime
    :ivar job_id: The job ID. Required.
    :vartype job_id: str
    :ivar last_updated_date_time: The last date time the job was updated. Required.
    :vartype last_updated_date_time: ~datetime.datetime
    :ivar status: Job Status. Required. Known values are: "notStarted", "running", "succeeded",
     "failed", "cancelled", "cancelling", and "partiallyCompleted".
    :vartype status: str or ~azure.ai.language.questionanswering.authoring.models.JobStatus
    :ivar errors: The errors encountered while executing the job.
    :vartype errors: list[~azure.core.ODataV4Format]
    """

    created_date_time: datetime.datetime = rest_field(
        name="createdDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The creation date time of the job. Required."""
    expiration_date_time: Optional[datetime.datetime] = rest_field(
        name="expirationDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The expiration date time of the job."""
    job_id: str = rest_field(name="jobId", visibility=["read"])
    """The job ID. Required."""
    last_updated_date_time: datetime.datetime = rest_field(
        name="lastUpdatedDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The last date time the job was updated. Required."""
    status: Union[str, "_models.JobStatus"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Job Status. Required. Known values are: \"notStarted\", \"running\", \"succeeded\", \"failed\",
     \"cancelled\", \"cancelling\", and \"partiallyCompleted\"."""
    errors: Optional[list[ODataV4Format]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The errors encountered while executing the job."""

    @overload
    def __init__(
        self,
        *,
        created_date_time: datetime.datetime,
        last_updated_date_time: datetime.datetime,
        status: Union[str, "_models.JobStatus"],
        expiration_date_time: Optional[datetime.datetime] = None,
        errors: Optional[list[ODataV4Format]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class QuestionAnsweringAuthoringUpdateSourcesJobState(_Model):  # pylint: disable=name-too-long
    """Job state represents the job metadata and any errors.

    :ivar created_date_time: The creation date time of the job. Required.
    :vartype created_date_time: ~datetime.datetime
    :ivar expiration_date_time: The expiration date time of the job.
    :vartype expiration_date_time: ~datetime.datetime
    :ivar job_id: The job ID. Required.
    :vartype job_id: str
    :ivar last_updated_date_time: The last date time the job was updated. Required.
    :vartype last_updated_date_time: ~datetime.datetime
    :ivar status: Job Status. Required. Known values are: "notStarted", "running", "succeeded",
     "failed", "cancelled", "cancelling", and "partiallyCompleted".
    :vartype status: str or ~azure.ai.language.questionanswering.authoring.models.JobStatus
    :ivar errors: The errors encountered while executing the job.
    :vartype errors: list[~azure.core.ODataV4Format]
    """

    created_date_time: datetime.datetime = rest_field(
        name="createdDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The creation date time of the job. Required."""
    expiration_date_time: Optional[datetime.datetime] = rest_field(
        name="expirationDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The expiration date time of the job."""
    job_id: str = rest_field(name="jobId", visibility=["read"])
    """The job ID. Required."""
    last_updated_date_time: datetime.datetime = rest_field(
        name="lastUpdatedDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The last date time the job was updated. Required."""
    status: Union[str, "_models.JobStatus"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Job Status. Required. Known values are: \"notStarted\", \"running\", \"succeeded\", \"failed\",
     \"cancelled\", \"cancelling\", and \"partiallyCompleted\"."""
    errors: Optional[list[ODataV4Format]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The errors encountered while executing the job."""

    @overload
    def __init__(
        self,
        *,
        created_date_time: datetime.datetime,
        last_updated_date_time: datetime.datetime,
        status: Union[str, "_models.JobStatus"],
        expiration_date_time: Optional[datetime.datetime] = None,
        errors: Optional[list[ODataV4Format]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class QuestionAnsweringProject(_Model):
    """Represents the project.

    :ivar project_name: Name of the project. Required.
    :vartype project_name: str
    :ivar description: Description of the project.
    :vartype description: str
    :ivar language: Language of the text records. This is BCP-47 representation of a language. For
     example, use "en" for English; "es" for Spanish etc. If not set, use "en" for
     English as default.
    :vartype language: str
    :ivar multilingual_resource: Resource enabled for multiple languages across projects or not.
    :vartype multilingual_resource: bool
    :ivar settings: Configurable settings of the Project.
    :vartype settings: ~azure.ai.language.questionanswering.authoring.models.ProjectSettings
    :ivar created_date_time: Project creation date-time.
    :vartype created_date_time: ~datetime.datetime
    :ivar last_modified_date_time: Represents the project last modified date-time.
    :vartype last_modified_date_time: ~datetime.datetime
    :ivar last_deployed_date_time: Represents the project last deployment date-time.
    :vartype last_deployed_date_time: ~datetime.datetime
    :ivar configure_semantic_ranking: Represents if semantic ranking is configured.
    :vartype configure_semantic_ranking: bool
    """

    project_name: str = rest_field(name="projectName", visibility=["read"])
    """Name of the project. Required."""
    description: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Description of the project."""
    language: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Language of the text records. This is BCP-47 representation of a language. For
     example, use \"en\" for English; \"es\" for Spanish etc. If not set, use \"en\" for
     English as default."""
    multilingual_resource: Optional[bool] = rest_field(
        name="multilingualResource", visibility=["read", "create", "update", "delete", "query"]
    )
    """Resource enabled for multiple languages across projects or not."""
    settings: Optional["_models.ProjectSettings"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Configurable settings of the Project."""
    created_date_time: Optional[datetime.datetime] = rest_field(
        name="createdDateTime", visibility=["read"], format="rfc3339"
    )
    """Project creation date-time."""
    last_modified_date_time: Optional[datetime.datetime] = rest_field(
        name="lastModifiedDateTime", visibility=["read"], format="rfc3339"
    )
    """Represents the project last modified date-time."""
    last_deployed_date_time: Optional[datetime.datetime] = rest_field(
        name="lastDeployedDateTime", visibility=["read"], format="rfc3339"
    )
    """Represents the project last deployment date-time."""
    configure_semantic_ranking: Optional[bool] = rest_field(
        name="configureSemanticRanking", visibility=["read", "create", "update", "delete", "query"]
    )
    """Represents if semantic ranking is configured."""

    @overload
    def __init__(
        self,
        *,
        description: Optional[str] = None,
        language: Optional[str] = None,
        multilingual_resource: Optional[bool] = None,
        settings: Optional["_models.ProjectSettings"] = None,
        configure_semantic_ranking: Optional[bool] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class RetrieveQnaRecord(_Model):
    """QnA Record with last updated date-time.

    :ivar id: Unique ID for the QnA. Required.
    :vartype id: int
    :ivar answer: Answer text.
    :vartype answer: str
    :ivar source: Source from which QnA was indexed e.g.
     `https://docs.microsoft.com/en-us/azure/cognitive-services/QnAMaker/FAQs
     <https://docs.microsoft.com/en-us/azure/cognitive-services/QnAMaker/FAQs>`_ .
    :vartype source: str
    :ivar questions: List of questions associated with the answer.
    :vartype questions: list[str]
    :ivar metadata: Metadata associated with the answer, useful to categorize or filter question
     answers.
    :vartype metadata: dict[str, str]
    :ivar dialog: Context of a QnA.
    :vartype dialog: ~azure.ai.language.questionanswering.authoring.models.QnaDialog
    :ivar active_learning_suggestions: List of Active Learning suggestions for the QnA.
    :vartype active_learning_suggestions:
     list[~azure.ai.language.questionanswering.authoring.models.SuggestedQuestionsCluster]
    :ivar last_updated_date_time: Date-time when the QnA was last updated.
    :vartype last_updated_date_time: ~datetime.datetime
    """

    id: int = rest_field(visibility=["read", "create"])
    """Unique ID for the QnA. Required."""
    answer: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Answer text."""
    source: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Source from which QnA was indexed e.g.
     `https://docs.microsoft.com/en-us/azure/cognitive-services/QnAMaker/FAQs
     <https://docs.microsoft.com/en-us/azure/cognitive-services/QnAMaker/FAQs>`_ ."""
    questions: Optional[list[str]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """List of questions associated with the answer."""
    metadata: Optional[dict[str, str]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Metadata associated with the answer, useful to categorize or filter question
     answers."""
    dialog: Optional["_models.QnaDialog"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Context of a QnA."""
    active_learning_suggestions: Optional[list["_models.SuggestedQuestionsCluster"]] = rest_field(
        name="activeLearningSuggestions", visibility=["read", "create", "update", "delete", "query"]
    )
    """List of Active Learning suggestions for the QnA."""
    last_updated_date_time: Optional[datetime.datetime] = rest_field(
        name="lastUpdatedDateTime", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """Date-time when the QnA was last updated."""

    @overload
    def __init__(
        self,
        *,
        id: int,  # pylint: disable=redefined-builtin
        answer: Optional[str] = None,
        source: Optional[str] = None,
        questions: Optional[list[str]] = None,
        metadata: Optional[dict[str, str]] = None,
        dialog: Optional["_models.QnaDialog"] = None,
        active_learning_suggestions: Optional[list["_models.SuggestedQuestionsCluster"]] = None,
        last_updated_date_time: Optional[datetime.datetime] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class SuggestedQuestion(_Model):
    """Question suggested by the Active Learning feature.

    :ivar question: Question suggested by the Active Learning feature.
    :vartype question: str
    :ivar user_suggested_count: The number of times the question was suggested explicitly by the
     user.
    :vartype user_suggested_count: int
    :ivar auto_suggested_count: The number of times the question was suggested automatically by the
     Active
     Learning algorithm.
    :vartype auto_suggested_count: int
    """

    question: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Question suggested by the Active Learning feature."""
    user_suggested_count: Optional[int] = rest_field(
        name="userSuggestedCount", visibility=["read", "create", "update", "delete", "query"]
    )
    """The number of times the question was suggested explicitly by the user."""
    auto_suggested_count: Optional[int] = rest_field(
        name="autoSuggestedCount", visibility=["read", "create", "update", "delete", "query"]
    )
    """The number of times the question was suggested automatically by the Active
     Learning algorithm."""

    @overload
    def __init__(
        self,
        *,
        question: Optional[str] = None,
        user_suggested_count: Optional[int] = None,
        auto_suggested_count: Optional[int] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class SuggestedQuestionsCluster(_Model):
    """Active Learning suggested questions cluster details.

    :ivar cluster_head: Question chosen as the head of suggested questions cluster by Active
     Learning
     clustering algorithm.
    :vartype cluster_head: str
    :ivar suggested_questions: List of all suggested questions for the QnA.
    :vartype suggested_questions:
     list[~azure.ai.language.questionanswering.authoring.models.SuggestedQuestion]
    """

    cluster_head: Optional[str] = rest_field(
        name="clusterHead", visibility=["read", "create", "update", "delete", "query"]
    )
    """Question chosen as the head of suggested questions cluster by Active Learning
     clustering algorithm."""
    suggested_questions: Optional[list["_models.SuggestedQuestion"]] = rest_field(
        name="suggestedQuestions", visibility=["read", "create", "update", "delete", "query"]
    )
    """List of all suggested questions for the QnA."""

    @overload
    def __init__(
        self,
        *,
        cluster_head: Optional[str] = None,
        suggested_questions: Optional[list["_models.SuggestedQuestion"]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class SynonymAssets(_Model):
    """Request payload for updating synonyms.

    :ivar value: The WordAlterations items on this page. Required.
    :vartype value: list[~azure.ai.language.questionanswering.authoring.models.WordAlterations]
    :ivar next_link: The link to the next page of items.
    :vartype next_link: str
    """

    value: list["_models.WordAlterations"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The WordAlterations items on this page. Required."""
    next_link: Optional[str] = rest_field(name="nextLink", visibility=["read", "create", "update", "delete", "query"])
    """The link to the next page of items."""

    @overload
    def __init__(
        self,
        *,
        value: list["_models.WordAlterations"],
        next_link: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class UpdateQnaRecord(_Model):
    """QnA record to be updated.

    :ivar op: Update operation type for assets. Required. Known values are: "add", "delete", and
     "replace".
    :vartype op: str or ~azure.ai.language.questionanswering.authoring.models.UpdateOperationKind
    :ivar value: QnA record. Required.
    :vartype value: ~azure.ai.language.questionanswering.authoring.models.QnaRecord
    """

    op: Union[str, "_models.UpdateOperationKind"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Update operation type for assets. Required. Known values are: \"add\", \"delete\", and
     \"replace\"."""
    value: "_models.QnaRecord" = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """QnA record. Required."""

    @overload
    def __init__(
        self,
        *,
        op: Union[str, "_models.UpdateOperationKind"],
        value: "_models.QnaRecord",
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class UpdateQnaSourceRecord(_Model):
    """Update source record.

    :ivar display_name: Friendly name of the Source.
    :vartype display_name: str
    :ivar source: Unique source identifier. Name of the file if it's a 'file' source; otherwise,
     the complete URL if it's a 'url' source. Required.
    :vartype source: str
    :ivar source_uri: URI location for the file or url. Required.
    :vartype source_uri: str
    :ivar source_kind: Supported source types. Required. Known values are: "file" and "url".
    :vartype source_kind: str or ~azure.ai.language.questionanswering.authoring.models.SourceKind
    :ivar content_structure_kind: Content structure type for sources. "unstructured"
    :vartype content_structure_kind: str or
     ~azure.ai.language.questionanswering.authoring.models.SourceContentStructureKind
    :ivar refresh: Boolean flag used to refresh data from the Source.
    :vartype refresh: bool
    """

    display_name: Optional[str] = rest_field(
        name="displayName", visibility=["read", "create", "update", "delete", "query"]
    )
    """Friendly name of the Source."""
    source: str = rest_field(visibility=["read", "create", "update"])
    """Unique source identifier. Name of the file if it's a 'file' source; otherwise,
     the complete URL if it's a 'url' source. Required."""
    source_uri: str = rest_field(name="sourceUri", visibility=["read", "create", "update", "delete", "query"])
    """URI location for the file or url. Required."""
    source_kind: Union[str, "_models.SourceKind"] = rest_field(
        name="sourceKind", visibility=["read", "create", "update", "delete", "query"]
    )
    """Supported source types. Required. Known values are: \"file\" and \"url\"."""
    content_structure_kind: Optional[Union[str, "_models.SourceContentStructureKind"]] = rest_field(
        name="contentStructureKind", visibility=["read", "create", "update", "delete", "query"]
    )
    """Content structure type for sources. \"unstructured\""""
    refresh: Optional[bool] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Boolean flag used to refresh data from the Source."""

    @overload
    def __init__(
        self,
        *,
        source: str,
        source_uri: str,
        source_kind: Union[str, "_models.SourceKind"],
        display_name: Optional[str] = None,
        content_structure_kind: Optional[Union[str, "_models.SourceContentStructureKind"]] = None,
        refresh: Optional[bool] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class UpdateSourceFiles(_Model):
    """Collection of files containing project assets that need to be imported.

    :ivar files: Collection of files where the fileName is required. Required.
    :vartype files: list[~azure.ai.language.questionanswering.authoring._utils.utils.FileType]
    :ivar file_operations: Collection of file operations to perform during update of sources.
     Required.
    :vartype file_operations:
     list[~azure.ai.language.questionanswering.authoring.models.UpdateSourceFromFileOperationRecord]
    """

    files: list[FileType] = rest_field(
        visibility=["read", "create", "update", "delete", "query"], is_multipart_file_input=True
    )
    """Collection of files where the fileName is required. Required."""
    file_operations: list["_models.UpdateSourceFromFileOperationRecord"] = rest_field(
        name="fileOperations", visibility=["read", "create", "update", "delete", "query"]
    )
    """Collection of file operations to perform during update of sources. Required."""

    @overload
    def __init__(
        self,
        *,
        files: list[FileType],
        file_operations: list["_models.UpdateSourceFromFileOperationRecord"],
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class UpdateSourceFromFileOperationRecord(_Model):
    """A single file operation for updating sources for a project. Used to add, remove or replace a
    source from a knowledgebase.

    :ivar operation: Update operation type for assets. Required. Known values are: "add", "delete",
     and "replace".
    :vartype operation: str or
     ~azure.ai.language.questionanswering.authoring.models.UpdateOperationKind
    :ivar file_display_name: Friendly name of the Source. If not provided, would be equal to
     fileSource.
    :vartype file_display_name: str
    :ivar file_name: This would be name of the file. This links to the fileName of the file being
     sent in multipart. Required.
    :vartype file_name: str
    :ivar refresh: Bool value for refresh.
    :vartype refresh: bool
    """

    operation: Union[str, "_models.UpdateOperationKind"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Update operation type for assets. Required. Known values are: \"add\", \"delete\", and
     \"replace\"."""
    file_display_name: Optional[str] = rest_field(
        name="fileDisplayName", visibility=["read", "create", "update", "delete", "query"]
    )
    """Friendly name of the Source. If not provided, would be equal to fileSource."""
    file_name: str = rest_field(name="fileName", visibility=["read", "create", "update", "delete", "query"])
    """This would be name of the file. This links to the fileName of the file being sent in multipart.
     Required."""
    refresh: Optional[bool] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Bool value for refresh."""

    @overload
    def __init__(
        self,
        *,
        operation: Union[str, "_models.UpdateOperationKind"],
        file_name: str,
        file_display_name: Optional[str] = None,
        refresh: Optional[bool] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class UpdateSourceRecord(_Model):
    """Source to be updated.

    :ivar op: Update operation type for assets. Required. Known values are: "add", "delete", and
     "replace".
    :vartype op: str or ~azure.ai.language.questionanswering.authoring.models.UpdateOperationKind
    :ivar value: Update source record. Required.
    :vartype value: ~azure.ai.language.questionanswering.authoring.models.UpdateQnaSourceRecord
    """

    op: Union[str, "_models.UpdateOperationKind"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Update operation type for assets. Required. Known values are: \"add\", \"delete\", and
     \"replace\"."""
    value: "_models.UpdateQnaSourceRecord" = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Update source record. Required."""

    @overload
    def __init__(
        self,
        *,
        op: Union[str, "_models.UpdateOperationKind"],
        value: "_models.UpdateQnaSourceRecord",
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class WordAlterations(_Model):
    """Collection of word alterations.

    :ivar alterations: Collection of word alterations. Required.
    :vartype alterations: list[str]
    """

    alterations: list[str] = rest_field(visibility=["read", "create", "update"])
    """Collection of word alterations. Required."""

    @overload
    def __init__(
        self,
        *,
        alterations: list[str],
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See LICENSE.txt in the project root for
# license information.
# -------------------------------------------------------------------------
import pytest
from unittest import mock
from socket import timeout as SocketTimeout

from urllib3.util import connection as urllib_connection
from urllib3.response import HTTPResponse as UrllibResponse
from urllib3.connection import HTTPConnection as UrllibConnection

from corehttp.rest import HttpRequest
from corehttp.transport.requests import RequestsTransport
from corehttp.runtime.pipeline import Pipeline
from corehttp.exceptions import ServiceResponseError, ServiceResponseTimeoutError, ServiceRequestTimeoutError


def test_already_close_with_with(caplog, port):
    transport = RequestsTransport()

    request = HttpRequest("GET", "http://localhost:{}/basic/string".format(port))

    with Pipeline(transport) as pipeline:
        pipeline.run(request)

    # This is now closed, new requests should fail
    with pytest.raises(ValueError) as err:
        transport.send(request)
    assert "HTTP transport has already been closed." in str(err)


def test_already_close_manually(caplog, port):
    transport = RequestsTransport()

    request = HttpRequest("GET", "http://localhost:{}/basic/string".format(port))

    transport.send(request)
    transport.close()

    # This is now closed, new requests should fail
    with pytest.raises(ValueError) as err:
        transport.send(request)
    assert "HTTP transport has already been closed." in str(err)


def test_close_too_soon_works_fine(caplog, port):
    transport = RequestsTransport()

    request = HttpRequest("GET", "http://localhost:{}/basic/string".format(port))

    transport.close()  # Never opened, should work fine
    result = transport.send(request)

    assert result  # No exception is good enough here


def test_requests_timeout_response(caplog, port):
    transport = RequestsTransport()

    request = HttpRequest("GET", f"http://localhost:{port}/basic/string")

    with mock.patch.object(UrllibConnection, "getresponse", side_effect=SocketTimeout) as mock_method:
        with pytest.raises(ServiceResponseTimeoutError) as err:
            transport.send(request, read_timeout=0.0001)

        with pytest.raises(ServiceResponseError) as err:
            transport.send(request, read_timeout=0.0001)

        stream_request = HttpRequest("GET", f"http://localhost:{port}/streams/basic")
        with pytest.raises(ServiceResponseTimeoutError) as err:
            transport.send(stream_request, stream=True, read_timeout=0.0001)

    stream_resp = transport.send(stream_request, stream=True)
    with mock.patch.object(UrllibResponse, "_handle_chunk", side_effect=SocketTimeout) as mock_method:
        with pytest.raises(ServiceResponseTimeoutError) as err:
            stream_resp.read()


def test_requests_timeout_request(caplog, port):
    transport = RequestsTransport()

    request = HttpRequest("GET", f"http://localhost:{port}/basic/string")

    with mock.patch.object(urllib_connection, "create_connection", side_effect=SocketTimeout) as mock_method:
        with pytest.raises(ServiceRequestTimeoutError) as err:
            transport.send(request, connection_timeout=0.0001)

        with pytest.raises(ServiceRequestTimeoutError) as err:
            transport.send(request, connection_timeout=0.0001)

        stream_request = HttpRequest("GET", f"http://localhost:{port}/streams/basic")
        with pytest.raises(ServiceRequestTimeoutError) as err:
            transport.send(stream_request, stream=True, connection_timeout=0.0001)



# The MIT License (MIT)
# Copyright (c) Microsoft Corporation. All rights reserved.

"""Async integration tests for the shared partition key range cache and PKRange namedtuple.

Async counterparts of test_shared_cache_integration.py, validating that the async
CosmosClient shares the routing map cache correctly, that clear_cache() works
transparently, and that PKRange namedtuples are compatible with all async operations.
"""

import unittest
import uuid

import pytest

import test_config
from azure.cosmos.aio import CosmosClient
from azure.cosmos._routing.aio.routing_map_provider import (
    PartitionKeyRangeCache,
    _shared_routing_map_cache,
    _shared_cache_lock,
)


@pytest.mark.cosmosEmulator
@pytest.mark.asyncio
class TestSharedCacheIntegrationAsync(unittest.IsolatedAsyncioTestCase):
    """Async integration tests requiring the Cosmos emulator."""

    host = test_config.TestConfig.host
    master_key = test_config.TestConfig.masterKey
    TEST_DATABASE_ID = test_config.TestConfig.TEST_DATABASE_ID
    TEST_CONTAINER_ID = test_config.TestConfig.TEST_MULTI_PARTITION_CONTAINER_ID

    async def asyncSetUp(self):
        self.client1 = CosmosClient(self.host, self.master_key)
        self.db = self.client1.get_database_client(self.TEST_DATABASE_ID)
        self.container = self.db.get_container_client(self.TEST_CONTAINER_ID)
        for i in range(20):
            await self.container.upsert_item(
                {"id": f"async-cache-item-{i}", "pk": f"pk-{i % 5}", "value": i}
            )

    async def asyncTearDown(self):
        for i in range(20):
            try:
                await self.container.delete_item(f"async-cache-item-{i}", partition_key=f"pk-{i % 5}")
            except Exception:
                pass
        await self.client1.close()
        # Release module-level shared routing-map state between async tests so
        # the test order cannot affect cache contents observed by a later test.
        # Clear ALL four shared-cache globals (not just the routing-map dict)
        # to keep refcount/lock state consistent.
        from azure.cosmos._routing.aio.routing_map_provider import (
            _shared_collection_locks,
            _shared_locks_locks,
            _shared_cache_refcounts,
        )
        with _shared_cache_lock:
            _shared_routing_map_cache.pop(self.host, None)
            _shared_collection_locks.pop(self.host, None)
            _shared_locks_locks.pop(self.host, None)
            _shared_cache_refcounts.pop(self.host, None)

    def _get_routing_provider(self, client):
        return client.client_connection._routing_map_provider

    def _get_cache_dict(self, client):
        return self._get_routing_provider(client)._collection_routing_map_by_item

    async def _populate_cache(self, client, container):
        """Force PK range cache population by directly calling the routing-map provider."""
        provider = self._get_routing_provider(client)
        await provider.get_routing_map(container.container_link, feed_options=None)

    async def test_multi_client_shared_cache_reads_async(self):
        """Async: Two clients to the same endpoint share the routing map."""
        async with CosmosClient(self.host, self.master_key) as client2:
            container2 = client2.get_database_client(self.TEST_DATABASE_ID).get_container_client(
                self.TEST_CONTAINER_ID)

            await self.container.read_item("async-cache-item-0", partition_key="pk-0")

            cache1 = self._get_cache_dict(self.client1)
            cache2 = self._get_cache_dict(client2)
            self.assertIs(cache1, cache2)

            result = await container2.read_item("async-cache-item-1", partition_key="pk-1")
            self.assertEqual(result["id"], "async-cache-item-1")

    async def test_multi_client_shared_cache_queries_async(self):
        """Async: Client2 uses cached routing map populated by client1 for queries."""
        async with CosmosClient(self.host, self.master_key) as client2:
            container2 = client2.get_database_client(self.TEST_DATABASE_ID).get_container_client(
                self.TEST_CONTAINER_ID)

            await self._populate_cache(self.client1, self.container)

            cache = self._get_cache_dict(self.client1)
            self.assertTrue(len(cache) > 0, "Cache should be populated after routing-map fetch")

            results = []
            async for item in container2.query_items(
                "SELECT * FROM c WHERE c.pk = 'pk-0'"
            ):
                results.append(item)
            self.assertTrue(len(results) > 0)

    async def test_clear_cache_triggers_repopulation_async(self):
        """Async: After clear_cache(), the next routing-map fetch transparently re-populates."""
        await self._populate_cache(self.client1, self.container)
        cache = self._get_cache_dict(self.client1)
        self.assertTrue(len(cache) > 0)

        provider = self._get_routing_provider(self.client1)
        provider.clear_cache()
        self.assertEqual(len(cache), 0)

        await self._populate_cache(self.client1, self.container)
        cache = self._get_cache_dict(self.client1)
        self.assertTrue(len(cache) > 0, "Cache should be re-populated after fetch")

    async def test_clear_cache_propagates_to_shared_clients_async(self):
        """Async: clear_cache() preserves dict identity for all sharing clients."""
        async with CosmosClient(self.host, self.master_key) as client2:
            container2 = client2.get_database_client(self.TEST_DATABASE_ID).get_container_client(
                self.TEST_CONTAINER_ID)

            await self.container.read_item("async-cache-item-0", partition_key="pk-0")

            self._get_routing_provider(self.client1).clear_cache()

            cache1 = self._get_cache_dict(self.client1)
            cache2 = self._get_cache_dict(client2)
            self.assertIs(cache1, cache2, "Both clients should reference the same dict after clear_cache")
            self.assertEqual(len(cache1), 0)

            result = await container2.read_item("async-cache-item-2", partition_key="pk-2")
            self.assertEqual(result["id"], "async-cache-item-2")

    async def test_different_endpoints_isolated_with_emulator_async(self):
        """Async: Emulator client cache is isolated from a different endpoint."""
        class DummyClient:
            url_connection = "https://other-async-account.documents.azure.com:443/"

        dummy_cache = PartitionKeyRangeCache(DummyClient())
        dummy_cache._collection_routing_map_by_item["dummy-coll"] = "dummy-data"

        await self.container.read_item("async-cache-item-0", partition_key="pk-0")
        emulator_cache = self._get_cache_dict(self.client1)

        self.assertNotIn("dummy-coll", emulator_cache)
        self.assertIn("dummy-coll", dummy_cache._collection_routing_map_by_item)

    async def test_pkrange_survives_full_crud_lifecycle_async(self):
        """Async: All CRUD operations work correctly with PKRange-based routing maps."""
        crud_id = f"async-crud-{uuid.uuid4()}"

        item = await self.container.create_item({"id": crud_id, "pk": "crud-pk", "data": "test"})
        self.assertEqual(item["id"], crud_id)

        read = await self.container.read_item(crud_id, partition_key="crud-pk")
        self.assertEqual(read["data"], "test")

        read["data"] = "updated"
        replaced = await self.container.replace_item(crud_id, read)
        self.assertEqual(replaced["data"], "updated")

        results = []
        async for r in self.container.query_items(
            f"SELECT * FROM c WHERE c.id = '{crud_id}'"
        ):
            results.append(r)
        self.assertEqual(len(results), 1)
        self.assertEqual(results[0]["data"], "updated")

        read["data"] = "upserted"
        upserted = await self.container.upsert_item(read)
        self.assertEqual(upserted["data"], "upserted")

        await self.container.delete_item(crud_id, partition_key="crud-pk")
        with self.assertRaises(Exception):
            await self.container.read_item(crud_id, partition_key="crud-pk")

        # Async point reads / writes don't always populate the routing-map
        # cache the way sync does (cf. _populate_cache helper). Drive a
        # routing-aware operation so the cache assertion below is meaningful.
        await self._populate_cache(self.client1, self.container)
        cache = self._get_cache_dict(self.client1)
        self.assertTrue(len(cache) > 0)

    async def test_pkrange_in_change_feed_async(self):
        """Async: Change feed operations work with PKRange-based routing maps."""
        cf_id = f"async-cf-{uuid.uuid4()}"
        await self.container.create_item({"id": cf_id, "pk": "cf-pk", "data": "change-feed-test"})

        results = []
        async for item in self.container.query_items_change_feed(
            start_time="Beginning",
            partition_key="cf-pk"
        ):
            results.append(item)
        self.assertTrue(len(results) > 0, "Change feed should return results")

        all_results = []
        async for item in self.container.query_items_change_feed(start_time="Beginning"):
            all_results.append(item)
        self.assertTrue(len(all_results) > 0, "Cross-partition change feed should return results")

        await self.container.delete_item(cf_id, partition_key="cf-pk")


if __name__ == "__main__":
    unittest.main()



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import pytest
import os
from devtools_testutils import recorded_by_proxy, AzureRecordedTestCase, RecordedTransport
from test_base import servicePreparer, fineTuningServicePreparer, modelsServicePreparer
from sample_executor import (
    AdditionalSampleTestDetail,
    SyncSampleExecutor,
    additionalSampleTests,
    get_sample_paths,
    SamplePathPasser,
)
from test_samples_helpers import get_sample_env_vars
from test_fine_tuning_samples_helpers import get_fine_tuning_sample_env_vars


class TestSamples(AzureRecordedTestCase):

    # To run this test with a specific sample, use:
    # pytest tests/samples/test_samples.py::TestSamples::test_agent_tools_samples[sample_agent_memory_search]
    @servicePreparer()
    # @additionalSampleTests(
    #     [
    #         AdditionalSampleTestDetail( # 2/28/2026 westus2 get 500
    #             test_id="sample_agent_azure_function",
    #             sample_filename="sample_agent_azure_function.py",
    #             env_vars={
    #                 "STORAGE_INPUT_QUEUE_NAME": "sanitized_input_queue_name",
    #                 "STORAGE_OUTPUT_QUEUE_NAME": "sanitized_output_queue_name",
    #                 "STORAGE_QUEUE_SERVICE_ENDPOINT": "sanitized_queue_service_endpoint",
    #             },
    #         ),
    #     ]
    # )
    @pytest.mark.parametrize(
        "sample_path",
        get_sample_paths(
            "agents/tools",
            samples_to_skip=[
                "sample_agent_file_search_structured_inputs.py",  # No issue to run. Just posepone recording.
                "sample_agent_code_interpreter_structured_inputs.py",  # No issue to run. Just posepone recording.
                "sample_agent_azure_function.py",  # In the list of additional sample tests above due to more parameters needed
                "sample_agent_computer_use.py",  # 400 BadRequestError: Invalid URI (URI string too long)
                "sample_agent_browser_automation.py",  # APITimeoutError: request timed out
                "sample_agent_openapi.py",  # 400 2/28/2026 validation/tool_user_error; failing weather GET curl call in OpenAPI tool
                "sample_agent_memory_search.py",  # Skipped until re-enabled and recorded on Foundry endpoint that supports the new versioning schema
            ],
        ),
    )
    @SamplePathPasser()
    @recorded_by_proxy(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    def test_agent_tools_samples(self, sample_path: str, **kwargs) -> None:
        env_vars = get_sample_env_vars(kwargs)
        executor = SyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        executor.execute()
        executor.validate_print_calls_by_llm()

    @pytest.mark.parametrize(
        "sample_path",
        get_sample_paths(
            "memories",
            samples_to_skip=[],
        ),
    )
    @servicePreparer()
    @SamplePathPasser()
    @recorded_by_proxy(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    # To run this test: pytest tests/samples/test_samples.py::TestSamples::test_memory_samples -s
    def test_memory_samples(self, sample_path: str, **kwargs) -> None:
        env_vars = get_sample_env_vars(kwargs)
        executor = SyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        executor.execute()
        executor.validate_print_calls_by_llm()

    @pytest.mark.parametrize(
        "sample_path",
        get_sample_paths(
            "agents",
            samples_to_skip=[
                "sample_external_agents_crud.py",  # Skipped until recordings are available.
                "sample_workflow_multi_agent.py",  # No issue to run.  Just postpone recording.
                "sample_workflow_multi_agent_with_mcp_approval.py",  # No issue to run.  Just postpone recording.
            ],
        ),
    )
    @servicePreparer()
    @SamplePathPasser()
    @recorded_by_proxy(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    def test_agents_samples(self, sample_path: str, **kwargs) -> None:
        env_vars = get_sample_env_vars(kwargs)
        executor = SyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        executor.execute()
        executor.validate_print_calls_by_llm()

    @pytest.mark.parametrize(
        "sample_path",
        get_sample_paths(
            "connections",
            samples_to_skip=[],
        ),
    )
    @servicePreparer()
    @SamplePathPasser()
    @recorded_by_proxy(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    def test_connections_samples(self, sample_path: str, **kwargs) -> None:
        kwargs = kwargs.copy()
        kwargs["connection_name"] = "mcp"
        env_vars = get_sample_env_vars(kwargs)
        executor = SyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        executor.execute()
        executor.validate_print_calls_by_llm()

    @pytest.mark.parametrize(
        "sample_path",
        get_sample_paths(
            "files",
            samples_to_skip=[],
        ),
    )
    @servicePreparer()
    @SamplePathPasser()
    @recorded_by_proxy(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    def test_files_samples(self, sample_path: str, **kwargs) -> None:
        env_vars = get_sample_env_vars(kwargs)
        executor = SyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        executor.execute()
        executor.validate_print_calls_by_llm()

    @pytest.mark.parametrize(
        "sample_path",
        get_sample_paths(
            "deployments",
            samples_to_skip=[],
        ),
    )
    @servicePreparer()
    @SamplePathPasser()
    @recorded_by_proxy(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    def test_deployments_samples(self, sample_path: str, **kwargs) -> None:
        env_vars = get_sample_env_vars(kwargs)
        executor = SyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        executor.execute()
        executor.validate_print_calls_by_llm()

    @pytest.mark.parametrize(
        "sample_path",
        get_sample_paths(
            "models",
            samples_to_test=[
                # `sample_models_basic.py` uses the `create()` helper which shells out
                # to AzCopy. AzCopy traffic isn't captured by the test proxy, so the
                # sample can't be replayed from a recording. Live re-recording is still
                # exercised via the standalone tests in `tests/models/`.
                "sample_models_create_and_poll.py",
            ],
        ),
    )
    @modelsServicePreparer()
    @SamplePathPasser()
    @recorded_by_proxy(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    def test_models_samples(self, sample_path: str, **kwargs) -> None:
        import secrets  # local import to avoid module-level dep

        env_vars = get_sample_env_vars(kwargs)
        # Foundry permanently reserves a `<name>/<version>` asset namespace even
        # after `models.delete`, so every live re-recording needs a unique name.
        # Sanitize back to a stable value in conftest so playback URLs match.
        suffix = secrets.token_hex(4) if self.is_live else "00000000"
        env_vars["MODEL_NAME"] = f"recsmplmdl{suffix}"
        env_vars["MODEL_VERSION"] = "1"
        executor = SyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        executor.execute()
        # `validate_print_calls_by_llm` is intentionally not called: it requires
        # an Azure OpenAI connection on the Foundry project, which the canary
        # project used for `.beta.models` recordings does not have. The sample
        # is still validated end-to-end by `executor.execute()` (any exception
        # fails the test).

    @servicePreparer()
    @additionalSampleTests(
        [
            AdditionalSampleTestDetail(
                test_id="sample_dataset_generation_job_simpleqna_with_prompt_source",
                sample_filename="sample_dataset_generation_job_simpleqna_with_prompt_source.py",
                env_vars={
                    "POLL_INTERVAL_SECONDS": "60",
                },
            ),
        ]
    )
    @pytest.mark.parametrize(
        "sample_path",
        get_sample_paths(
            "datasets",
            samples_to_skip=[
                "sample_dataset_generation_job_simpleqna_with_prompt_source.py",  # PR #47067: recording not yet available
                "sample_dataset_generation_job_traces_for_finetuning.py",  # PR #47067: recording not yet available
                "sample_dataset_generation_job_simpleqna_for_finetuning.py",  # PR #47067: recording not yet available
                "sample_dataset_generation_job_traces_for_evaluation.py",  # PR #47067: recording not yet available
                "sample_dataset_generation_job_simpleqna_with_agent_source.py",  # PR #47067: recording not yet available
                "sample_dataset_generation_job_simpleqna_with_file_source.py",  # PR #47067: recording not yet available
            ],
        ),
    )
    @SamplePathPasser()
    @recorded_by_proxy(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    def test_datasets_samples(self, sample_path: str, **kwargs) -> None:
        env_vars = get_sample_env_vars(kwargs)
        executor = SyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        executor.execute()
        executor.validate_print_calls_by_llm()

    @pytest.mark.parametrize(
        "sample_path",
        get_sample_paths(
            "chat_completions",
            samples_to_skip=[],
        ),
    )
    @servicePreparer()
    @SamplePathPasser()
    @recorded_by_proxy(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    def test_chat_completions_samples(self, sample_path: str, **kwargs) -> None:
        env_vars = get_sample_env_vars(kwargs)
        executor = SyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        executor.execute()
        executor.validate_print_calls_by_llm()

    @servicePreparer()
    @additionalSampleTests(
        [
            AdditionalSampleTestDetail(
                test_id="sample_create_hosted_agent_from_remote_build",
                sample_filename="sample_create_hosted_agent_from_code.py",
                env_vars={
                    "FOUNDRY_HOSTED_AGENT_REMOTE_BUILD": "true",
                },
            ),
        ]
    )
    @pytest.mark.parametrize(
        "sample_path",
        get_sample_paths(
            "hosted_agents",
            samples_to_skip=[
                "sample_routines_crud.py",  # Skipped due to service serialization issues
                "sample_routines_with_timer_trigger.py",  # Skipped due to service serialization issues
            ],
        ),
    )
    @SamplePathPasser()
    @recorded_by_proxy(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    def test_hosted_agents_samples(self, sample_path: str, **kwargs) -> None:
        if os.path.basename(sample_path).startswith("sample_create_hosted_agent") and not self.is_live:
            pytest.skip("sample_create_hosted_agent.py is skipped in replay mode due to RBAC complications.")
        env_vars = get_sample_env_vars(kwargs)
        executor = SyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        executor.execute()
        executor.validate_print_calls_by_llm()

    @pytest.mark.parametrize(
        "sample_path",
        get_sample_paths(
            "finetuning",
            samples_to_skip=[
                "sample_finetuning_reinforcement_job.py",  # 403 PermissionDeniedError: missing Microsoft.MachineLearningServices/workspaces/agents/action
                "sample_finetuning_dpo_job.py",  # 401 AuthenticationError: missing AIServices/agents/write data action
            ],
        ),
    )
    @fineTuningServicePreparer()
    @SamplePathPasser()
    @recorded_by_proxy(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    def test_finetuning_samples(self, sample_path: str, **kwargs) -> None:
        env_vars = get_fine_tuning_sample_env_vars(sample_path, kwargs)
        executor = SyncSampleExecutor(self, sample_path, env_vars=env_vars, **kwargs)
        executor.execute()
        executor.validate_print_calls_by_llm()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.deviceregistry.aio import DeviceRegistryMgmtClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestDeviceRegistryMgmtNamespaceDiscoveredAssetsOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(DeviceRegistryMgmtClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_namespace_discovered_assets_get(self, resource_group):
        response = await self.client.namespace_discovered_assets.get(
            resource_group_name=resource_group.name,
            namespace_name="str",
            discovered_asset_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_namespace_discovered_assets_begin_create_or_replace(self, resource_group):
        response = await (
            await self.client.namespace_discovered_assets.begin_create_or_replace(
                resource_group_name=resource_group.name,
                namespace_name="str",
                discovered_asset_name="str",
                resource={
                    "extendedLocation": {"name": "str", "type": "str"},
                    "location": "str",
                    "id": "str",
                    "name": "str",
                    "properties": {
                        "deviceRef": {"deviceName": "str", "endpointName": "str"},
                        "discoveryId": "str",
                        "version": 0,
                        "assetTypeRefs": ["str"],
                        "attributes": {"str": {}},
                        "datasets": [
                            {
                                "name": "str",
                                "dataPoints": [
                                    {
                                        "dataSource": "str",
                                        "name": "str",
                                        "dataPointConfiguration": "str",
                                        "lastUpdatedOn": "2020-02-20 00:00:00",
                                        "typeRef": "str",
                                    }
                                ],
                                "dataSource": "str",
                                "datasetConfiguration": "str",
                                "destinations": ["dataset_destination"],
                                "lastUpdatedOn": "2020-02-20 00:00:00",
                                "typeRef": "str",
                            }
                        ],
                        "defaultDatasetsConfiguration": "str",
                        "defaultDatasetsDestinations": ["dataset_destination"],
                        "defaultEventsConfiguration": "str",
                        "defaultEventsDestinations": ["event_destination"],
                        "defaultManagementGroupsConfiguration": "str",
                        "defaultStreamsConfiguration": "str",
                        "defaultStreamsDestinations": ["stream_destination"],
                        "description": "str",
                        "displayName": "str",
                        "documentationUri": "str",
                        "eventGroups": [
                            {
                                "name": "str",
                                "dataSource": "str",
                                "defaultDestinations": ["event_destination"],
                                "eventGroupConfiguration": "str",
                                "events": [
                                    {
                                        "name": "str",
                                        "dataSource": "str",
                                        "destinations": ["event_destination"],
                                        "eventConfiguration": "str",
                                        "lastUpdatedOn": "2020-02-20 00:00:00",
                                        "typeRef": "str",
                                    }
                                ],
                                "typeRef": "str",
                            }
                        ],
                        "externalAssetId": "str",
                        "hardwareRevision": "str",
                        "managementGroups": [
                            {
                                "name": "str",
                                "actions": [
                                    {
                                        "name": "str",
                                        "targetUri": "str",
                                        "actionConfiguration": "str",
                                        "actionType": "str",
                                        "lastUpdatedOn": "2020-02-20 00:00:00",
                                        "timeoutInSeconds": 0,
                                        "topic": "str",
                                        "typeRef": "str",
                                    }
                                ],
                                "dataSource": "str",
                                "defaultTimeoutInSeconds": 0,
                                "defaultTopic": "str",
                                "lastUpdatedOn": "2020-02-20 00:00:00",
                                "managementGroupConfiguration": "str",
                                "typeRef": "str",
                            }
                        ],
                        "manufacturer": "str",
                        "manufacturerUri": "str",
                        "model": "str",
                        "productCode": "str",
                        "provisioningState": "str",
                        "serialNumber": "str",
                        "softwareRevision": "str",
                        "streams": [
                            {
                                "name": "str",
                                "destinations": ["stream_destination"],
                                "lastUpdatedOn": "2020-02-20 00:00:00",
                                "streamConfiguration": "str",
                                "typeRef": "str",
                            }
                        ],
                    },
                    "systemData": {
                        "createdAt": "2020-02-20 00:00:00",
                        "createdBy": "str",
                        "createdByType": "str",
                        "lastModifiedAt": "2020-02-20 00:00:00",
                        "lastModifiedBy": "str",
                        "lastModifiedByType": "str",
                    },
                    "tags": {"str": "str"},
                    "type": "str",
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_namespace_discovered_assets_begin_update(self, resource_group):
        response = await (
            await self.client.namespace_discovered_assets.begin_update(
                resource_group_name=resource_group.name,
                namespace_name="str",
                discovered_asset_name="str",
                properties={
                    "properties": {
                        "assetTypeRefs": ["str"],
                        "attributes": {"str": {}},
                        "datasets": [
                            {
                                "name": "str",
                                "dataPoints": [
                                    {
                                        "dataSource": "str",
                                        "name": "str",
                                        "dataPointConfiguration": "str",
                                        "lastUpdatedOn": "2020-02-20 00:00:00",
                                        "typeRef": "str",
                                    }
                                ],
                                "dataSource": "str",
                                "datasetConfiguration": "str",
                                "destinations": ["dataset_destination"],
                                "lastUpdatedOn": "2020-02-20 00:00:00",
                                "typeRef": "str",
                            }
                        ],
                        "defaultDatasetsConfiguration": "str",
                        "defaultDatasetsDestinations": ["dataset_destination"],
                        "defaultEventsConfiguration": "str",
                        "defaultEventsDestinations": ["event_destination"],
                        "defaultManagementGroupsConfiguration": "str",
                        "defaultStreamsConfiguration": "str",
                        "defaultStreamsDestinations": ["stream_destination"],
                        "description": "str",
                        "deviceRef": {"deviceName": "str", "endpointName": "str"},
                        "discoveryId": "str",
                        "displayName": "str",
                        "documentationUri": "str",
                        "eventGroups": [
                            {
                                "name": "str",
                                "dataSource": "str",
                                "defaultDestinations": ["event_destination"],
                                "eventGroupConfiguration": "str",
                                "events": [
                                    {
                                        "name": "str",
                                        "dataSource": "str",
                                        "destinations": ["event_destination"],
                                        "eventConfiguration": "str",
                                        "lastUpdatedOn": "2020-02-20 00:00:00",
                                        "typeRef": "str",
                                    }
                                ],
                                "typeRef": "str",
                            }
                        ],
                        "hardwareRevision": "str",
                        "managementGroups": [
                            {
                                "name": "str",
                                "actions": [
                                    {
                                        "name": "str",
                                        "targetUri": "str",
                                        "actionConfiguration": "str",
                                        "actionType": "str",
                                        "lastUpdatedOn": "2020-02-20 00:00:00",
                                        "timeoutInSeconds": 0,
                                        "topic": "str",
                                        "typeRef": "str",
                                    }
                                ],
                                "dataSource": "str",
                                "defaultTimeoutInSeconds": 0,
                                "defaultTopic": "str",
                                "lastUpdatedOn": "2020-02-20 00:00:00",
                                "managementGroupConfiguration": "str",
                                "typeRef": "str",
                            }
                        ],
                        "manufacturer": "str",
                        "manufacturerUri": "str",
                        "model": "str",
                        "productCode": "str",
                        "serialNumber": "str",
                        "softwareRevision": "str",
                        "streams": [
                            {
                                "name": "str",
                                "destinations": ["stream_destination"],
                                "lastUpdatedOn": "2020-02-20 00:00:00",
                                "streamConfiguration": "str",
                                "typeRef": "str",
                            }
                        ],
                        "version": 0,
                    },
                    "tags": {"str": "str"},
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_namespace_discovered_assets_begin_delete(self, resource_group):
        response = await (
            await self.client.namespace_discovered_assets.begin_delete(
                resource_group_name=resource_group.name,
                namespace_name="str",
                discovered_asset_name="str",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_namespace_discovered_assets_list_by_resource_group(self, resource_group):
        response = self.client.namespace_discovered_assets.list_by_resource_group(
            resource_group_name=resource_group.name,
            namespace_name="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import pytest
import time
from unittest import TestCase

try:
    from unittest.mock import MagicMock, patch
except ImportError:  # python < 3.3
    from mock import MagicMock, patch  # type: ignore
import azure.communication.identity._shared.user_credential as user_credential
from azure.communication.identity._shared.user_credential import (
    CommunicationTokenCredential,
)
from azure.communication.identity._shared.utils import create_access_token
from azure.communication.identity._shared.utils import get_current_utc_as_int

from _shared.helper import (
    generate_token_with_custom_expiry_epoch,
    generate_token_with_custom_expiry,
)


class DummyToken:
    def __init__(self, token, expires_on):
        self.token = token
        self.expires_on = expires_on


class DummyTokenExchangeClient:
    def __init__(self, resource_endpoint, token_credential, scopes):
        self.resource_endpoint = resource_endpoint
        self.token_credential = token_credential
        self.scopes = scopes

    def exchange_entra_token(self):
        return DummyToken("dummy", 9999999999)
        

class DummyTokenExchangeClientSwitch:
    def __init__(self, resource_endpoint, token_credential, scopes):
        self.resource_endpoint = resource_endpoint
        self.token_credential = token_credential
        self.scopes = scopes
        self.call_count = 0

    def exchange_entra_token(self):
        self.call_count += 1
        if self.call_count == 1:
            return DummyToken("dummy_expired", int(time.time()) - 5 * 60)
        else:
            return DummyToken("dummy_valid", int(time.time()) + 60 * 60)


class TestCommunicationTokenCredential(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.sample_token = generate_token_with_custom_expiry_epoch(32503680000)  # 1/1/2030
        cls.expired_token = generate_token_with_custom_expiry_epoch(100)  # 1/1/1970

    def test_communicationtokencredential_decodes_token(self):
        credential = CommunicationTokenCredential(self.sample_token)
        access_token = credential.get_token()
        self.assertEqual(access_token.token, self.sample_token)

    def test_communicationtokencredential_throws_if_invalid_token(self):
        self.assertRaises(ValueError, lambda: CommunicationTokenCredential("foo.bar.tar"))

    def test_communicationtokencredential_throws_if_nonstring_token(self):
        self.assertRaises(TypeError, lambda: CommunicationTokenCredential(454))

    def test_communicationtokencredential_throws_if_proactive_refresh_enabled_without_token_refresher(
        self,
    ):
        with pytest.raises(ValueError) as err:
            CommunicationTokenCredential(self.sample_token, proactive_refresh=True)
        assert str(err.value) == "When 'proactive_refresh' is True, 'token_refresher' must not be None."
        with pytest.raises(ValueError) as err:
            CommunicationTokenCredential(self.sample_token, proactive_refresh=True, token_refresher=None)
        assert str(err.value) == "When 'proactive_refresh' is True, 'token_refresher' must not be None."

    def test_communicationtokencredential_static_token_returns_expired_token(self):
        credential = CommunicationTokenCredential(self.expired_token)
        self.assertEqual(credential.get_token().token, self.expired_token)

    def test_communicationtokencredential_token_expired_refresh_called(self):
        refresher = MagicMock(return_value=create_access_token(self.sample_token))
        credential = CommunicationTokenCredential(self.expired_token, token_refresher=refresher)
        access_token = credential.get_token()
        refresher.assert_called_once()
        self.assertEqual(access_token.token, self.sample_token)

    def test_communicationtokencredential_raises_if_refresher_returns_expired_token(
        self,
    ):
        refresher = MagicMock(return_value=create_access_token(self.expired_token))
        credential = CommunicationTokenCredential(self.expired_token, token_refresher=refresher)
        with self.assertRaises(ValueError):
            credential.get_token()
        self.assertEqual(refresher.call_count, 1)

    def test_uses_initial_token_as_expected(self):
        refresher = MagicMock(return_value=create_access_token(self.expired_token))
        credential = CommunicationTokenCredential(self.sample_token, token_refresher=refresher, proactive_refresh=True)
        access_token = credential.get_token()

        self.assertEqual(refresher.call_count, 0)
        self.assertEqual(access_token.token, self.sample_token)

    def test_proactive_refresher_should_not_be_called_before_specified_time(self):
        refresh_minutes = 10
        token_validity_minutes = 60
        start_timestamp = get_current_utc_as_int()
        skip_to_timestamp = start_timestamp + (refresh_minutes - 5) * 60

        initial_token = generate_token_with_custom_expiry(token_validity_minutes * 60)
        refreshed_token = generate_token_with_custom_expiry(2 * token_validity_minutes * 60)
        refresher = MagicMock(return_value=create_access_token(refreshed_token))

        with patch(
            user_credential.__name__ + "." + get_current_utc_as_int.__name__,
            return_value=skip_to_timestamp,
        ):
            credential = CommunicationTokenCredential(initial_token, token_refresher=refresher, proactive_refresh=True)
            access_token = credential.get_token()

            assert refresher.call_count == 0
            assert access_token.token == initial_token
            # check that next refresh is always scheduled
            assert credential._timer is None

    def test_proactive_refresher_should_be_called_after_specified_time(self):
        refresh_minutes = 10
        token_validity_minutes = 60
        start_timestamp = get_current_utc_as_int()
        skip_to_timestamp = start_timestamp + (token_validity_minutes - refresh_minutes + 5) * 60

        initial_token = generate_token_with_custom_expiry(token_validity_minutes * 60)
        refreshed_token = generate_token_with_custom_expiry(2 * token_validity_minutes * 60)
        refresher = MagicMock(return_value=create_access_token(refreshed_token))

        with patch(
            user_credential.__name__ + "." + get_current_utc_as_int.__name__,
            return_value=skip_to_timestamp,
        ):
            credential = CommunicationTokenCredential(initial_token, token_refresher=refresher, proactive_refresh=True)
            access_token = credential.get_token()

            assert refresher.call_count == 1
            assert access_token.token == refreshed_token
            # check that next refresh is always scheduled
            assert credential._timer is not None

    def test_proactive_refresher_keeps_scheduling_again(self):
        refresh_minutes = 10
        token_validity_minutes = 60
        expired_token = generate_token_with_custom_expiry(-5 * 60)
        skip_to_timestamp = get_current_utc_as_int() + (token_validity_minutes - refresh_minutes) * 60 + 1
        first_refreshed_token = create_access_token(generate_token_with_custom_expiry(token_validity_minutes * 60))
        last_refreshed_token = create_access_token(generate_token_with_custom_expiry(2 * token_validity_minutes * 60))
        refresher = MagicMock(side_effect=[first_refreshed_token, last_refreshed_token])

        credential = CommunicationTokenCredential(expired_token, token_refresher=refresher, proactive_refresh=True)
        access_token = credential.get_token()
        with patch(
            user_credential.__name__ + "." + get_current_utc_as_int.__name__,
            return_value=skip_to_timestamp,
        ):
            access_token = credential.get_token()

            assert refresher.call_count == 2
            assert access_token.token == last_refreshed_token.token
            # check that next refresh is always scheduled
            assert credential._timer is not None

    def test_fractional_backoff_applied_when_token_expiring(self):
        token_validity_seconds = 5 * 60
        expiring_token = generate_token_with_custom_expiry(token_validity_seconds)

        refresher = MagicMock(
            side_effect=[
                create_access_token(expiring_token),
                create_access_token(expiring_token),
            ]
        )

        credential = CommunicationTokenCredential(expiring_token, token_refresher=refresher, proactive_refresh=True)

        next_milestone = token_validity_seconds / 2

        with patch(
            user_credential.__name__ + "." + get_current_utc_as_int.__name__,
            return_value=(get_current_utc_as_int() + next_milestone),
        ):
            credential.get_token()
        assert refresher.call_count == 1
        next_milestone = next_milestone / 2
        assert credential._timer.interval == next_milestone

    def test_refresher_should_not_be_called_when_token_still_valid(self):
        generated_token = generate_token_with_custom_expiry(15 * 60)
        new_token = generate_token_with_custom_expiry(10 * 60)
        refresher = MagicMock(return_value=create_access_token(new_token))

        credential = CommunicationTokenCredential(generated_token, token_refresher=refresher, proactive_refresh=False)
        for _ in range(10):
            access_token = credential.get_token()

        refresher.assert_not_called()
        assert generated_token == access_token.token

    def test_exit_cancels_timer(self):
        refreshed_token = create_access_token(generate_token_with_custom_expiry(30 * 60))
        refresher = MagicMock(return_value=refreshed_token)
        credential = CommunicationTokenCredential(self.expired_token, token_refresher=refresher, proactive_refresh=True)
        credential.get_token()
        credential.close()
        assert credential._timer is None

    def test_exit_enter_scenario_throws_exception(self):
        refreshed_token = create_access_token(generate_token_with_custom_expiry(30 * 60))
        refresher = MagicMock(return_value=refreshed_token)
        credential = CommunicationTokenCredential(self.expired_token, token_refresher=refresher, proactive_refresh=True)
        credential.get_token()
        credential.close()
        assert credential._timer is None

        with pytest.raises(RuntimeError) as err:
            credential.get_token()
        assert str(err.value) == "An instance of CommunicationTokenCredential cannot be reused once it has been closed."

    def test_missing_fields_raises_value_error(self):
        # Only resource_endpoint provided
        with pytest.raises(ValueError) as excinfo:
            CommunicationTokenCredential(resource_endpoint="https://endpoint")
        assert "Missing: token_credential" in str(excinfo.value)

        # Only token_credential provided
        with pytest.raises(ValueError) as excinfo:
            CommunicationTokenCredential(token_credential=MagicMock())
        assert "Missing: resource_endpoint" in str(excinfo.value)

        # Only scopes provided
        with pytest.raises(ValueError) as excinfo:
            CommunicationTokenCredential(scopes=["scope"])
        assert "Missing: resource_endpoint, token_credential" in str(excinfo.value)

    def test_all_fields_present_calls_token_exchange(monkeypatch):
        # Patch TokenExchangeClient to our dummy
        with patch("azure.communication.identity._shared.user_credential.TokenExchangeClient", DummyTokenExchangeClient):
            cred = CommunicationTokenCredential(
                resource_endpoint="https://endpoint",
                token_credential=MagicMock(),
                scopes=["scope"]
            )
            token = cred.get_token()
            assert token.token == "dummy"
            assert token.expires_on == 9999999999

    def test_missing_scopes_calls_token_exchange(monkeypatch):
        # Patch TokenExchangeClient to our dummy
        with patch("azure.communication.identity._shared.user_credential.TokenExchangeClient", DummyTokenExchangeClient):
            cred = CommunicationTokenCredential(
                resource_endpoint="https://endpoint",
                token_credential=MagicMock()
            )
            token = cred.get_token()
            assert token.token == "dummy"
            assert token.expires_on == 9999999999

    def test_token_exchange_refreshes_from_expired_to_valid(monkeypatch):
        # Patch TokenExchangeClient to DummyTokenExchangeClientSwitch
        # First call returns expired token - when initializing the token
        with patch("azure.communication.identity._shared.user_credential.TokenExchangeClient", DummyTokenExchangeClientSwitch):
            cred = CommunicationTokenCredential(
                resource_endpoint="https://endpoint",
                token_credential=MagicMock(),
                scopes=["scope"]
            )
            # Second call will trigger a refresh and generate a valid token
            token2 = cred.get_token()
            assert token2.token == "dummy_valid"



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.azurearcdata.aio import AzureArcDataManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAzureArcDataManagementSqlManagedInstancesOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(AzureArcDataManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_sql_managed_instances_list(self, resource_group):
        response = self.client.sql_managed_instances.list(
            api_version="2022-03-01-preview",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_sql_managed_instances_list_by_resource_group(self, resource_group):
        response = self.client.sql_managed_instances.list_by_resource_group(
            resource_group_name=resource_group.name,
            api_version="2022-03-01-preview",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_sql_managed_instances_get(self, resource_group):
        response = await self.client.sql_managed_instances.get(
            resource_group_name=resource_group.name,
            sql_managed_instance_name="str",
            api_version="2022-03-01-preview",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_sql_managed_instances_begin_create(self, resource_group):
        response = await (
            await self.client.sql_managed_instances.begin_create(
                resource_group_name=resource_group.name,
                sql_managed_instance_name="str",
                sql_managed_instance={
                    "location": "str",
                    "properties": {
                        "activeDirectoryInformation": {"keytabInformation": {"keytab": "str"}},
                        "admin": "str",
                        "basicLoginInformation": {"password": "str", "username": "str"},
                        "clusterId": "str",
                        "dataControllerId": "str",
                        "endTime": "str",
                        "extensionId": "str",
                        "k8sRaw": {
                            "spec": {
                                "replicas": 0,
                                "scheduling": {
                                    "default": {"resources": {"limits": {"str": "str"}, "requests": {"str": "str"}}}
                                },
                            }
                        },
                        "lastUploadedDate": "2020-02-20 00:00:00",
                        "licenseType": "BasePrice",
                        "provisioningState": "str",
                        "startTime": "str",
                    },
                    "extendedLocation": {"name": "str", "type": "str"},
                    "id": "str",
                    "name": "str",
                    "sku": {
                        "name": "vCore",
                        "capacity": 0,
                        "dev": True,
                        "family": "str",
                        "size": "str",
                        "tier": "GeneralPurpose",
                    },
                    "systemData": {
                        "createdAt": "2020-02-20 00:00:00",
                        "createdBy": "str",
                        "createdByType": "str",
                        "lastModifiedAt": "2020-02-20 00:00:00",
                        "lastModifiedBy": "str",
                        "lastModifiedByType": "str",
                    },
                    "tags": {"str": "str"},
                    "type": "str",
                },
                api_version="2022-03-01-preview",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_sql_managed_instances_begin_delete(self, resource_group):
        response = await (
            await self.client.sql_managed_instances.begin_delete(
                resource_group_name=resource_group.name,
                sql_managed_instance_name="str",
                api_version="2022-03-01-preview",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_sql_managed_instances_update(self, resource_group):
        response = await self.client.sql_managed_instances.update(
            resource_group_name=resource_group.name,
            sql_managed_instance_name="str",
            parameters={"tags": {"str": "str"}},
            api_version="2022-03-01-preview",
        )

        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.compute import ComputeManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-compute
# USAGE
    python virtual_machine_extension_create_or_update_maximum_set_gen.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = ComputeManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.virtual_machine_extensions.begin_create_or_update(
        resource_group_name="rgcompute",
        vm_name="aaaaaaaaaaaaaaaaaaaaaaaa",
        vm_extension_name="aaaaaaaaaaaaa",
        extension_parameters={
            "location": "westus",
            "properties": {
                "autoUpgradeMinorVersion": True,
                "enableAutomaticUpgrade": True,
                "forceUpdateTag": "a",
                "instanceView": {
                    "name": "aaaaaaaaaaaaaaaaa",
                    "statuses": [
                        {
                            "code": "aaaaaaaaaaaaaaaaaaaaaaa",
                            "displayStatus": "aaaaaa",
                            "level": "Info",
                            "message": "a",
                            "time": "2021-11-30T12:58:26.522Z",
                        }
                    ],
                    "substatuses": [
                        {
                            "code": "aaaaaaaaaaaaaaaaaaaaaaa",
                            "displayStatus": "aaaaaa",
                            "level": "Info",
                            "message": "a",
                            "time": "2021-11-30T12:58:26.522Z",
                        }
                    ],
                    "type": "aaaaaaaaa",
                    "typeHandlerVersion": "aaaaaaaaaaaaaaaaaaaaaaaaaa",
                },
                "protectedSettings": {},
                "publisher": "extPublisher",
                "settings": {},
                "suppressFailures": True,
                "type": "extType",
                "typeHandlerVersion": "1.2",
            },
            "tags": {"key9183": "aa"},
        },
    ).result()
    print(response)


# x-ms-original-file: 2025-11-01/virtualMachineExamples/VirtualMachineExtension_CreateOrUpdate_MaximumSet_Gen.json
if __name__ == "__main__":
    main()



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
DESCRIPTION:
    This sample demonstrates how to emit human evaluation events from your
    application as OpenTelemetry custom events that land in the `customEvents`
    Application Insights table connected to your Microsoft Foundry project.

    Human evaluations capture signals that automated evaluators struggle with,
    such as tone, user satisfaction, and factual nuance. The helpers below emit
    the `gen_ai.evaluation.result` event shape used by Microsoft Foundry:

    * `emit_boolean_evaluation(...)` emits binary scores such as thumbs up/down.
    * `emit_5_point_ordinal_evaluation(...)` emits 1-5 ordered scores such as a
      Likert or star rating.

    Both public helpers call the shared `_emit_human_evaluation(...)` helper so
    the OpenTelemetry and Microsoft-specific attribute mapping stays consistent.
    This sample covers human evaluations submitted by end users of your
    application and correlates evaluation events to OpenAI Responses API response
    IDs when a response ID is provided.

    NOTE: Human evaluations are in preview and carry the risk of breaking
    changes.

USAGE:
    python sample_human_evaluations.py

    Before running the sample:

    pip install "azure-ai-projects>=2.0.0" python-dotenv azure-monitor-opentelemetry

    Set these environment variables with your own values:
    1) FOUNDRY_PROJECT_ENDPOINT - Required. The Azure AI Project endpoint, as
       found in the overview page of your Microsoft Foundry project. It has
       the form: https://<account_name>.services.ai.azure.com/api/projects/<project_name>.
"""

import json
import logging
from typing import Literal, Mapping, Optional

# NOTE: Azure SDK / Application Insights imports (azure-identity,
# azure-monitor-opentelemetry, azure-ai-projects, python-dotenv) are
# intentionally deferred into the `if __name__ == "__main__":` block at the
# bottom of this file. They are only needed when running the sample directly;
# the helper functions remain importable in test environments that do not install
# the full OTel stack. The trace-context imports are only needed when a saved
# trace ID and span ID are provided.

# `configure_azure_monitor` (called in __main__) installs an OpenTelemetry
# LoggingHandler on the root logger, so any standard Python `logging` call below
# this point flows through OTel to Application Insights as a log record. The
# `microsoft.custom_event.name` attribute routes the record to the
# `customEvents` table.
logger = logging.getLogger("human_evaluations")
logger.setLevel(logging.INFO)

EvaluationType = Literal["boolean", "ordinal"]
DesirableDirection = Literal["increase", "decrease"]


def _validate_hex_id(*, name: str, value: str, length: int) -> int:
    if len(value) != length:
        raise ValueError(f"{name} must be a {length}-character hexadecimal string.")

    try:
        parsed_value = int(value, 16)
    except ValueError as exc:
        raise ValueError(f"{name} must be a {length}-character hexadecimal string.") from exc

    if parsed_value == 0:
        raise ValueError(f"{name} cannot be all zeros.")

    return parsed_value


def _log_evaluation_event(
    *,
    attributes: Mapping[str, object],
    trace_id: Optional[str] = None,
    span_id: Optional[str] = None,
) -> None:
    if trace_id is None and span_id is None:
        logger.info("gen_ai.evaluation.result", extra=attributes)
        return
    if trace_id is None or span_id is None:
        raise ValueError("trace_id and span_id must be provided together.")

    trace_id_int = _validate_hex_id(name="trace_id", value=trace_id, length=32)
    span_id_int = _validate_hex_id(name="span_id", value=span_id, length=16)

    from opentelemetry import context as otel_context
    from opentelemetry import trace
    from opentelemetry.trace import (
        NonRecordingSpan,
        SpanContext,
        TraceFlags,
        TraceState,
    )

    span_context = SpanContext(
        trace_id=trace_id_int,
        span_id=span_id_int,
        is_remote=True,
        trace_flags=TraceFlags(TraceFlags.SAMPLED),
        trace_state=TraceState(),
    )
    token = otel_context.attach(trace.set_span_in_context(NonRecordingSpan(span_context)))
    try:
        logger.info("gen_ai.evaluation.result", extra=attributes)
    finally:
        otel_context.detach(token)


def _validate_score(
    *,
    score_value: float,
    min_value: float,
    max_value: float,
    evaluation_type: EvaluationType,
) -> float:
    score_value = float(score_value)
    if not (min_value <= score_value <= max_value):
        raise ValueError(
            f"score_value {score_value} is outside the allowed range "
            f"[{min_value}, {max_value}] for evaluation type '{evaluation_type}'."
        )
    if score_value != int(score_value):
        raise ValueError(
            f"score_value {score_value} must be a double-encoded integer value "
            f"for evaluation type '{evaluation_type}'."
        )
    return score_value


def _get_score_label(
    *,
    score_value: float,
    threshold: float,
    desirable_direction: DesirableDirection,
) -> Literal["pass", "fail"]:
    if desirable_direction == "increase":
        return "pass" if score_value >= threshold else "fail"
    if desirable_direction == "decrease":
        return "pass" if score_value <= threshold else "fail"
    raise ValueError(f"Unsupported desirable_direction: {desirable_direction!r}.")


def _emit_human_evaluation(
    *,
    evaluation_metric_name: str,
    score_value: float,
    evaluation_type: EvaluationType,
    min_value: float,
    max_value: float,
    threshold: float,
    desirable_direction: DesirableDirection,
    explanation: Optional[str] = None,
    response_id: Optional[str] = None,
    conversation_id: Optional[str] = None,
    project_resource_id: Optional[str] = None,
    enduser_id: Optional[str] = None,
    enduser_pseudo_id: Optional[str] = None,
    tags: Optional[Mapping[str, str]] = None,
    evaluation_id: Optional[str] = None,
    trace_id: Optional[str] = None,
    span_id: Optional[str] = None,
) -> None:
    score_value = _validate_score(
        score_value=score_value,
        min_value=min_value,
        max_value=max_value,
        evaluation_type=evaluation_type,
    )
    score_label = _get_score_label(
        score_value=score_value,
        threshold=threshold,
        desirable_direction=desirable_direction,
    )

    internal_properties = {
        "gen_ai.evaluation.threshold": str(threshold),
        "gen_ai.evaluation.min_value": str(min_value),
        "gen_ai.evaluation.max_value": str(max_value),
        "gen_ai.evaluation.desirable_direction": desirable_direction,
        "gen_ai.evaluation.type": evaluation_type,
    }
    if project_resource_id is not None:
        internal_properties["gen_ai.azure_ai_project.id"] = project_resource_id

    attributes = {
        "microsoft.custom_event.name": "gen_ai.evaluation.result",
        "gen_ai.evaluation.name": evaluation_metric_name,
        "gen_ai.evaluation.score.value": score_value,
        "gen_ai.evaluation.score.label": score_label,
        "microsoft.gen_ai.human_evaluation.source": "end_user",
        "microsoft.gen_ai.evaluation.actor.type": "human",
        "internal_properties": json.dumps(internal_properties),
    }
    if explanation is not None:
        attributes["gen_ai.evaluation.explanation"] = explanation
    if response_id is not None:
        attributes["gen_ai.response.id"] = response_id
    if conversation_id is not None:
        attributes["gen_ai.conversation.id"] = conversation_id
    if enduser_id is not None:
        attributes["enduser.id"] = enduser_id
    if enduser_pseudo_id is not None:
        attributes["enduser.pseudo.id"] = enduser_pseudo_id
    if tags:
        for tag_name, tag_value in tags.items():
            attributes[f"microsoft.gen_ai.evaluation.tags.{tag_name}"] = tag_value
    if evaluation_id is not None:
        attributes["microsoft.gen_ai.human_evaluation.id"] = evaluation_id

    _log_evaluation_event(attributes=attributes, trace_id=trace_id, span_id=span_id)


def emit_boolean_evaluation(
    *,
    evaluation_metric_name: str,
    passed: bool,
    explanation: Optional[str] = None,
    response_id: Optional[str] = None,
    conversation_id: Optional[str] = None,
    project_resource_id: Optional[str] = None,
    enduser_id: Optional[str] = None,
    enduser_pseudo_id: Optional[str] = None,
    tags: Optional[Mapping[str, str]] = None,
    evaluation_id: Optional[str] = None,
    trace_id: Optional[str] = None,
    span_id: Optional[str] = None,
) -> None:
    """Emit a boolean human evaluation event.

    Boolean evaluations are typically represented as thumbs up/down, yes/no, or
    pass/fail controls. `passed=True` emits a score of `1.0`; `passed=False`
    emits a score of `0.0`.

    Args:
        evaluation_metric_name: Name of the evaluated metric, such as
            `"task_completion"` or `"helpfulness"`.
        passed: Whether the human evaluation passed.
        explanation: Optional free-form explanation from the end user.
        response_id: Optional OpenAI Responses API response ID being evaluated.
        conversation_id: Optional conversation ID associated with the evaluation.
        project_resource_id: Optional ARM resource ID for the Foundry project.
        enduser_id: Optional signed-in end-user ID. This may contain PII.
        enduser_pseudo_id: Optional pseudonymous end-user ID.
        tags: Optional metadata associated with the evaluation.
        evaluation_id: Optional ID for the evaluation event itself.
        trace_id: Optional trace ID captured when the evaluated response was created.
        span_id: Optional span ID captured when the evaluated response was created.
    """
    _emit_human_evaluation(
        evaluation_metric_name=evaluation_metric_name,
        score_value=1.0 if passed else 0.0,
        evaluation_type="boolean",
        min_value=0.0,
        max_value=1.0,
        threshold=1.0,
        desirable_direction="increase",
        explanation=explanation,
        response_id=response_id,
        conversation_id=conversation_id,
        project_resource_id=project_resource_id,
        enduser_id=enduser_id,
        enduser_pseudo_id=enduser_pseudo_id,
        tags=tags,
        evaluation_id=evaluation_id,
        trace_id=trace_id,
        span_id=span_id,
    )


def emit_5_point_ordinal_evaluation(
    *,
    evaluation_metric_name: str,
    score_value: float,
    threshold: float = 3.0,
    explanation: Optional[str] = None,
    response_id: Optional[str] = None,
    conversation_id: Optional[str] = None,
    project_resource_id: Optional[str] = None,
    enduser_id: Optional[str] = None,
    enduser_pseudo_id: Optional[str] = None,
    tags: Optional[Mapping[str, str]] = None,
    evaluation_id: Optional[str] = None,
    trace_id: Optional[str] = None,
    span_id: Optional[str] = None,
) -> None:
    """Emit a 5-point ordinal human evaluation event.

    Ordinal 5-point evaluations use integer scores from `1.0` through `5.0`;
    scores at or above `3.0` are emitted with a `pass` label.

    Args:
        evaluation_metric_name: Name of the evaluated metric, such as
            `"relevance"` or `"helpfulness"`.
        score_value: Integer score from `1.0` through `5.0`.
        threshold: Score at or above this value is passing.
        explanation: Optional free-form explanation from the end user.
        response_id: Optional OpenAI Responses API response ID being evaluated.
        conversation_id: Optional conversation ID associated with the evaluation.
        project_resource_id: Optional ARM resource ID for the Foundry project.
        enduser_id: Optional signed-in end-user ID. This may contain PII.
        enduser_pseudo_id: Optional pseudonymous end-user ID.
        tags: Optional metadata associated with the evaluation.
        evaluation_id: Optional ID for the evaluation event itself.
        trace_id: Optional trace ID captured when the evaluated response was created.
        span_id: Optional span ID captured when the evaluated response was created.
    """
    threshold = float(threshold)
    if not 1.0 <= threshold <= 5.0:
        raise ValueError(f"threshold {threshold} is outside the allowed range [1.0, 5.0].")

    _emit_human_evaluation(
        evaluation_metric_name=evaluation_metric_name,
        score_value=score_value,
        evaluation_type="ordinal",
        min_value=1.0,
        max_value=5.0,
        threshold=threshold,
        desirable_direction="increase",
        explanation=explanation,
        response_id=response_id,
        conversation_id=conversation_id,
        project_resource_id=project_resource_id,
        enduser_id=enduser_id,
        enduser_pseudo_id=enduser_pseudo_id,
        tags=tags,
        evaluation_id=evaluation_id,
        trace_id=trace_id,
        span_id=span_id,
    )


if __name__ == "__main__":
    import os

    from azure.ai.projects import AIProjectClient
    from azure.identity import DefaultAzureCredential
    from azure.monitor.opentelemetry import configure_azure_monitor
    from dotenv import load_dotenv

    load_dotenv()

    endpoint = os.environ["FOUNDRY_PROJECT_ENDPOINT"]

    with (
        DefaultAzureCredential() as credential,
        AIProjectClient(endpoint=endpoint, credential=credential) as project_client,
    ):
        # Pull the Application Insights connection string attached to your Foundry
        # project and wire OpenTelemetry up to it. All `logger.info(...)` calls
        # below will be exported to Application Insights.
        connection_string = project_client.telemetry.get_application_insights_connection_string()

        configure_azure_monitor(connection_string=connection_string)

        # Optional: derive the Foundry Project's resource id from any connection.
        # The endpoint URL alone only gives us the account + project names, but
        # every Connection's `id` is a full ARM path ending with /connections/<name>.
        any_connection = next(iter(project_client.connections.list()), None)
        project_resource_id = any_connection.id.rsplit("/connections/", 1)[0] if any_connection else None

        # Sample trace and span IDs for demonstration purposes.
        trace_id = "4bf92f3577b34da6a3ce929d0e0e4736"
        span_id = "00f067aa0ba902b7"

        # Example 1: an anonymous end user gives a thumbs up on task completion.
        emit_boolean_evaluation(
            evaluation_metric_name="task_completion",
            passed=True,
            explanation="The agent provided accurate weather information as requested.",
            response_id="resp_64904952b20872620069f8d600779c81908f58b0a3be090ef0",
            conversation_id="conv_5j66UpCpwteGg4YSxUnt7lPY",
            project_resource_id=project_resource_id,
            enduser_pseudo_id="sess_123456",
            tags={"subscription_tier": "free_plan"},
            evaluation_id="0b27be45-cd65-4671-ab08-c3eafd4c9613",
            trace_id=trace_id,
            span_id=span_id,
        )
        print("Emitted boolean human evaluation event.")

        # Example 2: a signed-in end user rates relevance on a 5-point scale.
        emit_5_point_ordinal_evaluation(
            evaluation_metric_name="relevance",
            score_value=4.0,
            explanation=(
                "The agent's response is relevant to the query, providing useful "
                "information that addresses the user's intent."
            ),
            response_id="resp_64904952b20872620069f8d600779c81908f58b0a3be090ef0",
            conversation_id="conv_5j66UpCpwteGg4YSxUnt7lPY",
            project_resource_id=project_resource_id,
            enduser_id="oid:241964ad-a8db-4318-9f2e-5a7dc1f05349",
            tags={"department": "marketing"},
            evaluation_id="69d937a7-32e2-412e-97c9-119e2d282723",
            trace_id=trace_id,
            span_id=span_id,
        )
        print("Emitted 5-point ordinal human evaluation event.")



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.deviceregistry import DeviceRegistryMgmtClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-deviceregistry
# USAGE
    python create_or_replace_namespace_device_edge_x509.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = DeviceRegistryMgmtClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.namespace_devices.begin_create_or_replace(
        resource_group_name="myResourceGroup",
        namespace_name="adr-namespace-gbk0925-n01",
        device_name="namespace-device-on-edge",
        resource={
            "extendedLocation": {
                "name": "/subscriptions/00000000-0000-0000-0000-000000000000/resourcegroups/myResourceGroup/providers/microsoft.extendedlocation/customlocations/location1",
                "type": "CustomLocation",
            },
            "location": "West Europe",
            "properties": {
                "attributes": {"deviceCategory": 16, "deviceOwner": "OT", "deviceType": "OPCUAServers"},
                "enabled": True,
                "endpoints": {
                    "inbound": {
                        "theV1OPCUAEndpoint": {
                            "address": "opc.tcp://192.168.86.23:51211/UA/SampleServer",
                            "authentication": {
                                "method": "Certificate",
                                "x509Credentials": {"certificateSecretName": "cert-secret"},
                            },
                            "endpointType": "microsoft.opcua",
                            "version": "2",
                        },
                        "theV2OPCUAEndpoint": {
                            "address": "opc.tcp://192.168.86.23:51211/UA/SampleServer",
                            "authentication": {
                                "method": "Certificate",
                                "x509Credentials": {"certificateSecretName": "cert-secret"},
                            },
                            "endpointType": "microsoft.opcua",
                            "trustSettings": {"trustList": "trust-secret-reference"},
                            "version": "2",
                        },
                    }
                },
                "externalDeviceId": "unique-edge-device-identifier",
            },
        },
    ).result()
    print(response)


# x-ms-original-file: 2026-03-01-preview/CreateOrReplace_NamespaceDevice_Edge_x509.json
if __name__ == "__main__":
    main()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.billing.aio import BillingManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestBillingManagementPoliciesOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(BillingManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_policies_get_by_customer_at_billing_account(self, resource_group):
        response = await self.client.policies.get_by_customer_at_billing_account(
            billing_account_name="str",
            customer_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_policies_begin_create_or_update_by_customer_at_billing_account(self, resource_group):
        response = await (
            await self.client.policies.begin_create_or_update_by_customer_at_billing_account(
                billing_account_name="str",
                customer_name="str",
                parameters={
                    "id": "str",
                    "name": "str",
                    "properties": {
                        "viewCharges": "str",
                        "policies": [{"name": "str", "policyType": "str", "scope": "str", "value": "str"}],
                        "provisioningState": "str",
                    },
                    "systemData": {
                        "createdAt": "2020-02-20 00:00:00",
                        "createdBy": "str",
                        "createdByType": "str",
                        "lastModifiedAt": "2020-02-20 00:00:00",
                        "lastModifiedBy": "str",
                        "lastModifiedByType": "str",
                    },
                    "tags": {"str": "str"},
                    "type": "str",
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_policies_begin_create_or_update_by_customer(self, resource_group):
        response = await (
            await self.client.policies.begin_create_or_update_by_customer(
                billing_account_name="str",
                billing_profile_name="str",
                customer_name="str",
                parameters={
                    "id": "str",
                    "name": "str",
                    "properties": {
                        "viewCharges": "str",
                        "policies": [{"name": "str", "policyType": "str", "scope": "str", "value": "str"}],
                        "provisioningState": "str",
                    },
                    "systemData": {
                        "createdAt": "2020-02-20 00:00:00",
                        "createdBy": "str",
                        "createdByType": "str",
                        "lastModifiedAt": "2020-02-20 00:00:00",
                        "lastModifiedBy": "str",
                        "lastModifiedByType": "str",
                    },
                    "tags": {"str": "str"},
                    "type": "str",
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_policies_get_by_customer(self, resource_group):
        response = await self.client.policies.get_by_customer(
            billing_account_name="str",
            billing_profile_name="str",
            customer_name="str",
            policy_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_policies_get_by_billing_profile(self, resource_group):
        response = await self.client.policies.get_by_billing_profile(
            billing_account_name="str",
            billing_profile_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_policies_begin_create_or_update_by_billing_profile(self, resource_group):
        response = await (
            await self.client.policies.begin_create_or_update_by_billing_profile(
                billing_account_name="str",
                billing_profile_name="str",
                parameters={
                    "id": "str",
                    "name": "str",
                    "properties": {
                        "enterpriseAgreementPolicies": {
                            "accountOwnerViewCharges": "str",
                            "authenticationType": "str",
                            "departmentAdminViewCharges": "str",
                        },
                        "invoiceSectionLabelManagement": "str",
                        "marketplacePurchases": "str",
                        "policies": [{"name": "str", "policyType": "str", "scope": "str", "value": "str"}],
                        "provisioningState": "str",
                        "reservationPurchases": "str",
                        "savingsPlanPurchases": "str",
                        "viewCharges": "str",
                    },
                    "systemData": {
                        "createdAt": "2020-02-20 00:00:00",
                        "createdBy": "str",
                        "createdByType": "str",
                        "lastModifiedAt": "2020-02-20 00:00:00",
                        "lastModifiedBy": "str",
                        "lastModifiedByType": "str",
                    },
                    "tags": {"str": "str"},
                    "type": "str",
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_policies_get_by_billing_account(self, resource_group):
        response = await self.client.policies.get_by_billing_account(
            billing_account_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_policies_begin_create_or_update_by_billing_account(self, resource_group):
        response = await (
            await self.client.policies.begin_create_or_update_by_billing_account(
                billing_account_name="str",
                parameters={
                    "id": "str",
                    "name": "str",
                    "properties": {
                        "enterpriseAgreementPolicies": {
                            "accountOwnerViewCharges": "str",
                            "authenticationType": "str",
                            "departmentAdminViewCharges": "str",
                        },
                        "marketplacePurchases": "str",
                        "policies": [{"name": "str", "policyType": "str", "scope": "str", "value": "str"}],
                        "provisioningState": "str",
                        "reservationPurchases": "str",
                        "savingsPlanPurchases": "str",
                    },
                    "systemData": {
                        "createdAt": "2020-02-20 00:00:00",
                        "createdBy": "str",
                        "createdByType": "str",
                        "lastModifiedAt": "2020-02-20 00:00:00",
                        "lastModifiedBy": "str",
                        "lastModifiedByType": "str",
                    },
                    "tags": {"str": "str"},
                    "type": "str",
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_policies_get_by_subscription(self, resource_group):
        response = await self.client.policies.get_by_subscription()

        # please add some check logic here by yourself
        # ...



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.chaos import ChaosManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-chaos
# USAGE
    python scenario_configurations_create_or_update.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = ChaosManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.scenario_configurations.begin_create_or_update(
        resource_group_name="exampleRG",
        workspace_name="exampleWorkspace",
        scenario_name="12345678-1234-1234-1234-123456789012",
        scenario_configuration_name="config-5678-9012-3456-789012345678",
        resource={
            "properties": {
                "exclusions": {
                    "resources": [
                        "/subscriptions/6b052e15-03d3-4f17-b2e1-be7f07588291/resourceGroups/exampleRG/providers/Microsoft.Compute/virtualMachines/protectedVM"
                    ],
                    "tags": [{"key": "environment", "value": "production"}],
                    "types": ["Microsoft.Compute/virtualMachineScaleSets"],
                },
                "filters": {"locations": ["eastus"], "zones": ["1"]},
                "parameters": [
                    {"key": "duration", "value": "PT10M"},
                    {
                        "key": "targetResourceIds",
                        "value": '["/subscriptions/6b052e15-03d3-4f17-b2e1-be7f07588291/resourceGroups/exampleRG/providers/Microsoft.Compute/virtualMachines/vm1","/subscriptions/6b052e15-03d3-4f17-b2e1-be7f07588291/resourceGroups/exampleRG/providers/Microsoft.Compute/virtualMachines/vm2"]',
                    },
                ],
                "scenarioId": "/subscriptions/6b052e15-03d3-4f17-b2e1-be7f07588291/resourceGroups/exampleRG/providers/Microsoft.Chaos/workspaces/exampleWorkspace/scenarios/12345678-1234-1234-1234-123456789012",
            }
        },
    ).result()
    print(response)


# x-ms-original-file: 2026-05-01-preview/ScenarioConfigurations_CreateOrUpdate.json
if __name__ == "__main__":
    main()
